(function () {'use strict';

function _interopDefault (ex) { return (ex && (typeof ex === 'object') && 'default' in ex) ? ex['default'] : ex; }

var electron = require('electron');
var path = require('path');
var path__default = _interopDefault(path);
var beakerCore = require('@beaker/core');
var rpc = require('pauls-electron-rpc');
var electronUpdater = require('electron-updater');
var os = _interopDefault(require('os'));
var fs = require('fs');
var fs__default = _interopDefault(fs);
var slugify = _interopDefault(require('slugify'));
var jetpack = _interopDefault(require('fs-jetpack'));
var ICO = _interopDefault(require('icojs'));
var toIco = _interopDefault(require('to-ico'));
var emitStream = _interopDefault(require('emit-stream'));
var Events = _interopDefault(require('events'));
var LRU = _interopDefault(require('lru'));
var debounce = _interopDefault(require('lodash.debounce'));
var _isEqual = _interopDefault(require('lodash.isequal'));
var errorPage = _interopDefault(require('@beaker/core/lib/error-page'));
var _throttle = _interopDefault(require('lodash.throttle'));
var parseDatURL = _interopDefault(require('parse-dat-url'));
var _get = _interopDefault(require('lodash.get'));
var _pick = _interopDefault(require('lodash.pick'));
var normalizeURL = _interopDefault(require('normalize-url'));
var url = require('url');
var url__default = _interopDefault(url);
var beakerErrorConstants = require('beaker-error-constants');
var mime = _interopDefault(require('mime'));
var speedometer = _interopDefault(require('speedometer'));
var parseDataURL = _interopDefault(require('data-urls'));
var pda = _interopDefault(require('pauls-dat-api'));
var _beaker_core_lib_strings = require('@beaker/core/lib/strings');
var _flattenDeep = _interopDefault(require('lodash.flattendeep'));
var isAccelerator = _interopDefault(require('electron-is-accelerator'));
var equals = _interopDefault(require('keyboardevents-areequal'));
var keyboardeventFromElectronAccelerator = require('keyboardevent-from-electron-accelerator');
var _beaker_core_lib_const = require('@beaker/core/lib/const');
var crypto = _interopDefault(require('crypto'));
var https = _interopDefault(require('https'));
var querystring = _interopDefault(require('querystring'));
var ms = _interopDefault(require('ms'));
var concat = _interopDefault(require('concat-stream'));
var osName = _interopDefault(require('os-name'));
var natUpnp = _interopDefault(require('nat-upnp'));
var childProcess = require('child_process');
var once = _interopDefault(require('once'));
var intoStream = _interopDefault(require('into-stream'));
var dgram = _interopDefault(require('dgram'));

function defaultBrowsingSessionState () {
  return {
    windows: [ defaultWindowState() ],
    cleanExit: true
  }
}

function defaultWindowState () {
  // HACK
  // for some reason, electron.screen comes back null sometimes
  // not sure why, shouldn't be happening
  // check for existence for now, see #690
  // -prf
  const screen$$1 = require('electron').screen;
  var bounds = screen$$1 ? screen$$1.getPrimaryDisplay().bounds : {width: 800, height: 600};
  var width = Math.max(800, Math.min(1800, bounds.width - 50));
  var height = Math.max(600, Math.min(1200, bounds.height - 50));
  var minWidth = 400;
  var minHeight = 300;
  return {
    x: (bounds.width - width) / 2,
    y: (bounds.height - height) / 2,
    width,
    height,
    minWidth,
    minHeight,
    pages: defaultPageState()
  }
}

function defaultPageState () {
  return [ 'beaker://start' ]
}

const SNAPSHOT_PATH = 'shell-window-state.json';

// exported api
// =

class SessionWatcher {
  static get emptySnapshot () {
    return {
      windows: [],
      // We set this to false by default and clean this up when the session
      // exits. If we ever open up a snapshot and this isn't cleaned up assume
      // there was a crash
      cleanExit: false
    }
  }

  constructor (userDataDir) {
    this.userDataDir = userDataDir;
    this.snapshot = SessionWatcher.emptySnapshot;
    this.recording = true;
  }

  startRecording () { this.recording = true; }
  stopRecording () { this.recording = false; }

  watchWindow (win, initialState) {
    let state = initialState;
    this.snapshot.windows.push(state);
    let watcher = new WindowWatcher(win, initialState);

    watcher.on('change', (nextState) => {
      if (this.recording) {
        let { windows } = this.snapshot;
        let i = windows.indexOf(state);
        state = windows[i] = nextState;
        this.writeSnapshot();
      }
    });

    watcher.on('remove', () => {
      if (this.recording) {
        let { windows } = this.snapshot;
        let i = windows.indexOf(state);
        this.snapshot.windows = windows.splice(1, i);
        this.writeSnapshot();
      }
      watcher.removeAllListeners();
    });
  }

  exit () {
    this.snapshot.cleanExit = true;
    this.writeSnapshot();
  }

  writeSnapshot () {
    this.userDataDir.write(SNAPSHOT_PATH, this.snapshot, { atomic: true });
  }
}

// internal methods
// =

class WindowWatcher extends Events {
  constructor (win, initialState) {
    super();
    this.handleClosed = this.handleClosed.bind(this);
    this.handlePagesUpdated = this.handlePagesUpdated.bind(this);
    this.handlePositionChange = this.handlePositionChange.bind(this);

    // right now this class trusts that the initial state is correctly formed by this point
    this.snapshot = initialState;
    this.winId = win.id;
    win.on('closed', this.handleClosed);
    win.on('resize', debounce(this.handlePositionChange, 1000));
    win.on('moved', this.handlePositionChange);
    win.on('custom-pages-updated', this.handlePagesUpdated);
  }

  getWindow () {
    return electron.BrowserWindow.fromId(this.winId)
  }

  // handlers

  handleClosed () {
    var win = electron.BrowserWindow.fromId(this.winId);
    if (win) win.removeListener('custom-pages-updated', this.handlePagesUpdated);
    this.emit('remove');
  }

  handlePagesUpdated (pages) {
    if (_isEqual(pages, this.snapshot.pages)) return
    this.snapshot.pages = (pages && pages.length) ? pages : defaultPageState();
    this.emit('change', this.snapshot);
  }

  handlePositionChange () {
    Object.assign(this.snapshot, getCurrentPosition(this.getWindow()));
    this.emit('change', this.snapshot);
  }
}

function getCurrentPosition (win) {
  var position = win.getPosition();
  var size = win.getSize();
  return {
    x: position[0],
    y: position[1],
    width: size[0],
    height: size[1]
  }
}

var viewsRPCManifest = {
  createEventStream: 'readable',
  refreshState: 'promise',
  getState: 'promise',
  getTabState: 'promise',
  createTab: 'promise',
  loadURL: 'promise',
  closeTab: 'promise',
  setActiveTab: 'promise',
  reorderTab: 'promise',
  showTabContextMenu: 'promise',
  showLocationBarContextMenu: 'promise',
  goBack: 'promise',
  goForward: 'promise',
  stop: 'promise',
  reload: 'promise',
  resetZoom: 'promise',
  toggleLiveReloading: 'promise',
  showInpageFind: 'promise',
  hideInpageFind: 'promise',
  setInpageFindString: 'promise',
  moveInpageFind: 'promise',
  showLocationBar: 'promise',
  hideLocationBar: 'promise',
  runLocationBarCmd: 'promise',
  showMenu: 'promise',
  toggleMenu: 'promise',
  focusShellWindow: 'promise',
  onFaviconLoadSuccess: 'promise',
  onFaviconLoadError: 'promise'
};

const sitedataDb = beakerCore.dbs.sitedata;

const ZOOM_STEP = 0.5;

// exported api
// =

async function setZoomFromSitedata (view, origin) {
  // load zoom from sitedata
  origin = origin || view.origin;
  if (!origin) return

  var v = await sitedataDb.get(origin, 'zoom');
  view.zoom = +v || 0;
  view.emitUpdateState();
  view.webContents.setZoomLevel(view.zoom);
}

function setZoom (view, z) {
  // clamp
  if (z > 4.5) z = 4.5;
  if (z < -3) z = -3;

  // update
  view.zoom = z;
  view.webContents.setZoomLevel(view.zoom);
  view.emitUpdateState();

  // persist to sitedata
  var origin = view.origin;
  if (!origin) return
  sitedataDb.set(view.url, 'zoom', view.zoom);

  // update all pages at the origin
  getAll(view.browserWindow).forEach(v => {
    if (v !== view && v.origin === origin) {
      v.zoom = z;
    }
  });
}

function zoomIn (view) {
  setZoom(view, view.zoom + ZOOM_STEP);
}

function zoomOut (view) {
  setZoom(view, view.zoom - ZOOM_STEP);
}

function zoomReset (view) {
  setZoom(view, 0);
}

var shellMenusRPCManifest = {
  close: 'promise',
  createWindow: 'promise',
  createTab: 'promise',
  loadURL: 'promise',
  resizeSelf: 'promise',
  showInpageFind: 'promise'
};

/**
 * Shell Menus
 *
 * NOTES
 * - There can only ever be one Shell Menu window for a given browser window
 * - Shell Menu windows are created with each browser window and then shown/hidden as needed
 * - The Shell Menu window contains the UIs for multiple menus and swaps between them as needed
 * - When unfocused, the Shell Menu window is hidden (it's meant to act as a popup menu)
 */

// globals
// =

var events = new Events();
var windows = {}; // map of {[parentWindow.id] => BrowserWindow}

// exported api
// =

function setup$5 (parentWindow) {
  var win = windows[parentWindow.id] = new electron.BrowserWindow({
    parent: parentWindow,
    frame: false,
    resizable: false,
    maximizable: false,
    show: false,
    fullscreenable: false,
    webPreferences: {
      defaultEncoding: 'utf-8',
      preload: path__default.join(__dirname, 'shell-menus.build.js')
    }
  });
  win.webContents.on('console-message', (e, level, message) => {
    console.log('Shell-Menus window says:', message);
  });
  win.loadURL('beaker://shell-menus/');
  win.on('blur', () => hide(parentWindow));
}

function destroy (parentWindow) {
  if (get(parentWindow)) {
    get(parentWindow).close();
    delete windows[parentWindow.id];
  }
}

function get (parentWindow) {
  return windows[parentWindow.id]
}

function reposition (parentWindow) {
  var win = get(parentWindow);
  if (win) {
    var parentBounds = parentWindow.getBounds();
    if (win.menuId === 'browser') {
      win.setBounds({
        x: parentBounds.x + parentBounds.width - 245,
        y: parentBounds.y + 74,
        width: 240,
        height: 466
      });
    } else if (win.menuId === 'page') {
      win.setBounds({
        x: parentBounds.x + win.boundsOpt.right - 220,
        y: parentBounds.y + win.boundsOpt.top,
        width: 220,
        height: 110
      });
    } else if (win.menuId === 'location') {
      win.setBounds({
        x: parentBounds.x + win.boundsOpt.x,
        y: parentBounds.y + win.boundsOpt.y,
        width: win.boundsOpt.width,
        height: 310
      });
    } else if (win.menuId === 'bookmark') {
      win.setBounds({
        x: parentBounds.x + win.boundsOpt.right - 300,
        y: parentBounds.y + win.boundsOpt.top,
        width: 300,
        height: 250
      });
    } else if (win.menuId === 'peers') {
      win.setBounds({
        x: parentBounds.x + win.boundsOpt.right - 300,
        y: parentBounds.y + win.boundsOpt.top,
        width: 300,
        height: 118
      });
    } else if (win.menuId === 'donate') {
      win.setBounds({
        x: parentBounds.x + win.boundsOpt.right - 320,
        y: parentBounds.y + win.boundsOpt.top,
        width: 350,
        height: 90
      });
    } else if (win.menuId === 'local-path') {
      win.setBounds({
        x: parentBounds.x + win.boundsOpt.right - 160,
        y: parentBounds.y + win.boundsOpt.top,
        width: 160,
        height: 96
      });
    } else if (win.menuId === 'site-info') {
      win.setBounds({
        x: parentBounds.x + 100,
        y: parentBounds.y + 74,
        width: 450,
        height: 300
      });
    }
  }
}

async function toggle (parentWindow, menuId, opts) {
  var win = get(parentWindow);
  if (win) {
    if (win.isVisible()) {
      return hide(parentWindow)
    } else {
      return show(parentWindow, menuId, opts)
    }
  }
}

async function show (parentWindow, menuId, opts) {
  var win = get(parentWindow);
  if (win) {
    win.menuId = menuId;
    win.boundsOpt = opts && opts.bounds;
    reposition(parentWindow);

    var params = opts && opts.params ? opts.params : {};
    await win.webContents.executeJavaScript(`openMenu('${menuId}', ${JSON.stringify(params)})`);
    win.show();

    // await till hidden
    await new Promise(resolve => {
      events.once('hide', resolve);
    });
  }
}

function hide (parentWindow) {
  if (get(parentWindow)) {
    get(parentWindow).hide();
    events.emit('hide');
  }
}

// rpc api
// =

rpc.exportAPI('background-process-shell-menus', shellMenusRPCManifest, {
  async close () {
    hide(getParentWindow(this.sender));
  },

  async createWindow (url$$1) {
    createShellWindow();
  },

  async createTab (url$$1) {
    var win = getParentWindow(this.sender);
    hide(win); // always close the menu
    create$$1(win, url$$1, {setActive: true});
  },

  async loadURL (url$$1) {
    var win = getParentWindow(this.sender);
    hide(win); // always close the menu
    getActive(win).loadURL(url$$1);
  },

  async resizeSelf (dimensions) {
    var win = electron.BrowserWindow.fromWebContents(this.sender);
    if (process.platform === 'win32') {
      // on windows, add space for the border
      if (dimensions.width) dimensions.width += 2;
      if (dimensions.height) dimensions.height += 2;
    }
    win.setBounds(dimensions);
  },

  async showInpageFind () {
    var win = getParentWindow(this.sender);
    var view = getActive(win);
    if (view) view.showInpageFind();
  }
});

// internal methods
// =

function getParentWindow (sender) {
  return electron.BrowserWindow.fromWebContents(sender).getParentWindow()
}

var shellMenusSubwindow = Object.freeze({
	setup: setup$5,
	destroy: destroy,
	get: get,
	reposition: reposition,
	toggle: toggle,
	show: show,
	hide: hide
});

var locationBarRPCManifest = {
  close: 'promise',
  createTab: 'promise',
  loadURL: 'promise',
  resizeSelf: 'promise'
};

/**
 * Location Bar
 *
 * NOTES
 * - There can only ever be one Location Bar window for a given browser window
 * - Location Bar windows are created with each browser window and then shown/hidden as needed
 * - When unfocused, the Location Bar window is hidden (it's meant to act as a popup menu)
 */
const WIDTH = 800;
const HEIGHT = 310;

// globals
// =

var events$1 = new Events();
var windows$1 = {}; // map of {[parentWindow.id] => BrowserWindow}

// exported api
// =

function setup$6 (parentWindow) {
  var win = windows$1[parentWindow.id] = new electron.BrowserWindow({
    width: WIDTH,
    height: HEIGHT,
    parent: parentWindow,
    frame: false,
    resizable: false,
    maximizable: false,
    show: false,
    fullscreenable: false,
    acceptFirstMouse: true,
    webPreferences: {
      defaultEncoding: 'utf-8',
      preload: path__default.join(__dirname, 'location-bar.build.js')
    }
  });
  win.webContents.on('console-message', (e, level, message) => {
    console.log('Location-Bar window says:', message);
  });
  win.loadURL('beaker://location-bar/');
  win.on('blur', () => hide$1(parentWindow));
}

function destroy$1 (parentWindow) {
  if (get$1(parentWindow)) {
    get$1(parentWindow).close();
    delete windows$1[parentWindow.id];
  }
}

function get$1 (parentWindow) {
  return windows$1[parentWindow.id]
}

function reposition$1 (parentWindow) {
  var win = get$1(parentWindow);
  if (win && win.boundsOpt) {
    var parentBounds = parentWindow.getBounds();
    win.setBounds({
      x: parentBounds.x + win.boundsOpt.x,
      y: parentBounds.y + win.boundsOpt.y,
      width: win.boundsOpt.width,
      height: 310
    });
  }
}

async function show$1 (parentWindow, opts) {
  var win = get$1(parentWindow);
  if (win) {
    win.boundsOpt = opts && opts.bounds;
    reposition$1(parentWindow);
    win.webContents.executeJavaScript(`setup()`);
    win.showInactive();

    // await till hidden
    await new Promise(resolve => {
      events$1.once('hide', resolve);
    });
  }
}

function hide$1 (parentWindow) {
  if (get$1(parentWindow)) {
    get$1(parentWindow).hide();
  }
}

async function runCmd (parentWindow, cmd, opts) {
  var win = get$1(parentWindow);
  if (win) {
    if (!win.isVisible()) {
      if (cmd === 'set-value') {
        // show first
        show$1(parentWindow, opts);
      } else {
        return
      }
    }
    return win.webContents.executeJavaScript(`command("${cmd}", ${JSON.stringify(opts)})`)
  }
}

// rpc api
// =

rpc.exportAPI('background-process-location-bar', locationBarRPCManifest, {
  async close () {
    hide$1(getParentWindow$1(this.sender));
  },

  async createTab (url$$1) {
    var win = getParentWindow$1(this.sender);
    hide$1(win); // always close the location bar
    create$$1(win, url$$1, {setActive: true});
  },

  async loadURL (url$$1) {
    var win = getParentWindow$1(this.sender);
    hide$1(win); // always close the location bar
    getActive(win).loadURL(url$$1);
    win.webContents.send('command', 'unfocus-location'); // we have to manually unfocus the location bar
  },

  async resizeSelf (dimensions) {
    var win = electron.BrowserWindow.fromWebContents(this.sender);
    if (process.platform === 'win32') {
      // on windows, add space for the border
      if (dimensions.width) dimensions.width += 2;
      if (dimensions.height) dimensions.height += 2;
    }
    win.setBounds(dimensions);
  }
});

// internal methods
// =

function getParentWindow$1 (sender) {
  return electron.BrowserWindow.fromWebContents(sender).getParentWindow()
}

var locationBarSubwindow = Object.freeze({
	setup: setup$6,
	destroy: destroy$1,
	get: get$1,
	reposition: reposition$1,
	show: show$1,
	hide: hide$1,
	runCmd: runCmd
});

/**
 * Status Bar
 *
 * NOTES
 * - There can only ever be one Status Bar window for a given browser window
 * - Status Bar windows are created with each browser window and then shown/hidden as needed
 */
const WIDTH$1 = 400;
const HEIGHT$1 = 24;
const IS_DARWIN = process.platform === 'darwin';

// globals
// =

var windows$2 = {}; // map of {[parentWindow.id] => BrowserWindow}

// exported api
// =

function setup$7 (parentWindow) {
  var win = windows$2[parentWindow.id] = new electron.BrowserWindow({
    width: WIDTH$1,
    height: HEIGHT$1,
    parent: parentWindow,
    frame: false,
    transparent: true,
    resizable: false,
    maximizable: false,
    show: false,
    fullscreenable: false,
    hasShadow: false,
    focusable: false,
    webPreferences: {
      defaultEncoding: 'utf-8'
    }
  });
  win.setIgnoreMouseEvents(true);
  win.loadFile('status-bar.html');
  if (IS_DARWIN) win.setOpacity(0);
  win.webContents.executeJavaScript(`set(false)`);
}

function destroy$2 (parentWindow) {
  if (get$2(parentWindow)) {
    get$2(parentWindow).close();
    delete windows$2[parentWindow.id];
  }
}

function get$2 (parentWindow) {
  return windows$2[parentWindow.id]
}

function reposition$2 (parentWindow) {
  var win = get$2(parentWindow);
  if (win) {
    var {x, y, height} = parentWindow.getBounds();
    win.setBounds({x, y: y + height - HEIGHT$1});
  }
}

function show$2 (parentWindow) {
  var win = get$2(parentWindow);
  if (win) {
    reposition$2(parentWindow);
    if (!win.isVisible()) win.showInactive();
    if (IS_DARWIN) win.setOpacity(1);
  }
}

function hide$2 (parentWindow) {
  var win = get$2(parentWindow);
  if (win) {
    if (IS_DARWIN) win.setOpacity(0);
  }
}

function set (parentWindow, value) {
  var win = get$2(parentWindow);
  if (win) {
    if (value) show$2(parentWindow);
    win.webContents.executeJavaScript(`set(${JSON.stringify(value)})`);
    if (!value) hide$2(parentWindow);
  }
}

var statusBarSubwindow = Object.freeze({
	setup: setup$7,
	destroy: destroy$2,
	get: get$2,
	reposition: reposition$2,
	show: show$2,
	hide: hide$2,
	set: set
});

var permPromptRPCManifest = {
  createTab: 'promise',
  resizeSelf: 'promise'
};

/**
 * Perm Prompt
 *
 * NOTES
 * - Perm Prompt windows are created as-needed, and desroyed when not in use
 * - Perm Prompt windows are attached to individual BrowserView instances
 * - Perm Prompt windows are shown and hidden based on whether its owning BrowserView is visible
 */

// globals
// =

var windows$3 = {}; // map of {[parentView.id] => BrowserWindow}

// exported api
// =

function setup$8 (parentWindow) {
}

function destroy$3 (parentWindow) {
  // destroy all under this window
  for (let view of getAll(parentWindow)) {
    if (view.id in windows$3) {
      windows$3[view.id].close();
      delete windows$3[view.id];
    }
  }
}

function reposition$3 (parentWindow) {
  // reposition all under this window
  for (let view of getAll(parentWindow)) {
    if (view.id in windows$3) {
      setBounds(windows$3[view.id], parentWindow);
    }
  }
}

async function create$1 (parentWindow, parentView, params) {
  // make sure a prompt window doesnt already exist
  if (parentView.id in windows$3) {
    return false // abort
  }

  // create the window
  var win = windows$3[parentView.id] = new electron.BrowserWindow({
    parent: parentWindow,
    frame: false,
    resizable: false,
    maximizable: false,
    show: true,
    fullscreenable: false,
    webPreferences: {
      defaultEncoding: 'utf-8',
      preload: path__default.join(__dirname, 'perm-prompt.build.js')
    }
  });
  setBounds(win, parentWindow);
  win.webContents.on('console-message', (e, level, message) => {
    console.log('Perm-Prompt window says:', message);
  });
  win.loadURL('beaker://perm-prompt/');

  // run the prompt flow
  var decision = false;
  try {
    decision = await win.webContents.executeJavaScript(`runPrompt(${JSON.stringify(params)})`);
  } catch (e) {
    console.error('Failed to run permission prompt', e);
  }

  // destroy the window
  win.close();
  delete windows$3[parentView.id];
  return decision
}

function get$3 (parentView) {
  return windows$3[parentView.id]
}

function show$3 (parentView) {
  if (parentView.id in windows$3) {
    windows$3[parentView.id].show();
  }
}

function hide$3 (parentView) {
  if (parentView.id in windows$3) {
    windows$3[parentView.id].hide();
  }
}

function close (parentView) {
  if (parentView.id in windows$3) {
    windows$3[parentView.id].close();
    delete windows$3[parentView.id];
  }
}

// rpc api
// =

rpc.exportAPI('background-process-perm-prompt', permPromptRPCManifest, {
  async createTab (url$$1) {
    var win = getParentWindow$2(this.sender);
    create$$1(win, url$$1, {setActive: true});
  },

  async resizeSelf (dimensions) {
    var win = electron.BrowserWindow.fromWebContents(this.sender);
    var [width, height] = win.getSize();
    win.setSize(dimensions.width || width, dimensions.height || height);
  }
});

// internal methods
// =

function setBounds (win, parentWindow) {
  var parentBounds = parentWindow.getBounds();
  var width = 300;
  var height = 118;
  if (process.platform === 'win32') {
    // on windows, add space for the border
    width += 2;
    height += 3;
  }
  win.setBounds({
    x: parentBounds.x + 100,
    y: parentBounds.y + 74,
    width,
    height
  });
}

function getParentWindow$2 (sender) {
  return electron.BrowserWindow.fromWebContents(sender).getParentWindow()
}

var permPromptSubwindow = Object.freeze({
	setup: setup$8,
	destroy: destroy$3,
	reposition: reposition$3,
	create: create$1,
	get: get$3,
	show: show$3,
	hide: hide$3,
	close: close
});

var modalsRPCManifest = {
  createTab: 'promise',
  resizeSelf: 'promise'
};

/**
 * Modal
 *
 * NOTES
 * - Modal windows are created as-needed, and desroyed when not in use
 * - Modal windows are attached to individual BrowserView instances
 * - Modal windows are shown and hidden based on whether its owning BrowserView is visible
 */

// globals
// =

var windows$4 = {}; // map of {[parentView.id] => BrowserWindow}

// exported api
// =

function setup$9 (parentWindow) {
  // listen for the basic auth login event
  electron.app.on('login', async function (e, webContents$$1, request, authInfo, cb) {
    e.preventDefault(); // default is to cancel the auth; prevent that
    var res = await create$2(webContents$$1, 'basic-auth', authInfo);
    cb(res.username, res.password);
  });
}

function destroy$4 (parentWindow) {
  // destroy all under this window
  for (let view of getAll(parentWindow)) {
    if (view.id in windows$4) {
      windows$4[view.id].close();
      delete windows$4[view.id];
    }
  }
}

function reposition$4 (parentWindow) {
  // reposition all under this window
  for (let view of getAll(parentWindow)) {
    if (view.id in windows$4) {
      setBounds$1(windows$4[view.id], parentWindow);
    }
  }
}

async function create$2 (webContents$$1, modalName, params = {}) {
  // find parent window
  var parentWindow = electron.BrowserWindow.fromWebContents(webContents$$1);
  var parentView = electron.BrowserView.fromWebContents(webContents$$1);
  if (parentView && !parentWindow) {
    // if there's no window, then a web page created the prompt
    // use its containing window
    parentWindow = findContainingWindow(parentView);
  } else if (!parentView) {
    // if there's no view, then the shell window created the prompt
    // attach it to the active view
    parentView = getActive(parentWindow);
  }

  // make sure a prompt window doesnt already exist
  if (parentView.id in windows$4) {
    throw new beakerErrorConstants.ModalActiveError()
  }

  // create the window
  var win = windows$4[parentView.id] = new electron.BrowserWindow({
    parent: parentWindow,
    frame: false,
    resizable: false,
    maximizable: false,
    show: true,
    fullscreenable: false,
    webPreferences: {
      defaultEncoding: 'utf-8',
      preload: path__default.join(__dirname, 'modals.build.js')
    }
  });
  setBounds$1(win, parentWindow);
  win.webContents.on('console-message', (e, level, message) => {
    console.log('Modals window says:', message);
  });
  win.loadURL('beaker://modals/');

  // run the modal flow
  var result;
  var err;
  try {
    result = await win.webContents.executeJavaScript(`runModal("${modalName}", ${JSON.stringify(params)})`);
  } catch (e) {
    err = e;
  }

  // destroy the window
  win.close();
  delete windows$4[parentView.id];

  // return/throw
  if (err) throw err
  return result
}

function get$4 (parentView) {
  return windows$4[parentView.id]
}

function show$4 (parentView) {
  if (parentView.id in windows$4) {
    windows$4[parentView.id].show();
  }
}

function hide$4 (parentView) {
  if (parentView.id in windows$4) {
    windows$4[parentView.id].hide();
  }
}

function close$1 (parentView) {
  if (parentView.id in windows$4) {
    windows$4[parentView.id].close();
    delete windows$4[parentView.id];
  }
}

// rpc api
// =

rpc.exportAPI('background-process-modals', modalsRPCManifest, {
  async createTab (url$$1) {
    var win = getParentWindow$3(this.sender);
    create$$1(win, url$$1, {setActive: true});
  },

  async resizeSelf (dimensions) {
    var win = electron.BrowserWindow.fromWebContents(this.sender);
    var [width, height] = win.getSize();
    width = dimensions.width || width;
    height = dimensions.height || height;
    if (process.platform === 'win32') {
      // on windows, add space for the border
      width += 2;
      height += 2;
    }
    var parentBounds = win.getParentWindow().getBounds();
    win.setBounds({
      x: parentBounds.x + Math.round(parentBounds.width / 2) - Math.round(width / 2), // centered
      y: parentBounds.y + 74,
      width,
      height
    });
  }
});

// internal methods
// =

function setBounds$1 (win, parentWindow) {
  var parentBounds = parentWindow.getBounds();
  win.setBounds({
    x: parentBounds.x + Math.round(parentBounds.width / 2) - 250, // centered
    y: parentBounds.y + 74,
    width: 500,
    height: 300
  });
}

function getParentWindow$3 (sender) {
  return electron.BrowserWindow.fromWebContents(sender).getParentWindow()
}

var modalsSubwindow = Object.freeze({
	setup: setup$9,
	destroy: destroy$4,
	reposition: reposition$4,
	create: create$2,
	get: get$4,
	show: show$4,
	hide: hide$4,
	close: close$1
});

/* globals DatArchive */

var PERMS = {
  js: {
    persist: true,
    idempotent: true,
    alwaysDisallow: false,
    requiresRefresh: true,
    experimental: false
  },
  network: {
    persist: true,
    idempotent: true,
    alwaysDisallow: false,
    requiresRefresh: true,
    experimental: false
  },
  createDat: {
    persist: false,
    idempotent: false,
    alwaysDisallow: false,
    requiresRefresh: false,
    experimental: false
  },
  modifyDat: {
    persist: 'allow', // dont persist 'deny'
    idempotent: true,
    alwaysDisallow: false,
    requiresRefresh: false,
    experimental: false
  },
  deleteDat: {
    persist: false,
    idempotent: false,
    alwaysDisallow: false,
    requiresRefresh: false,
    experimental: false
  },
  media: {
    persist: true,
    idempotent: true,
    alwaysDisallow: false,
    requiresRefresh: false,
    experimental: false
  },
  geolocation: {
    persist: false,
    idempotent: true,
    alwaysDisallow: true, // NOTE geolocation is disabled, right now
    requiresRefresh: false,
    experimental: false
  },
  notifications: {
    persist: true,
    idempotent: true,
    alwaysDisallow: false,
    requiresRefresh: false,
    experimental: false
  },
  midiSysex: {
    persist: false,
    idempotent: true,
    alwaysDisallow: false,
    requiresRefresh: false,
    experimental: false
  },
  pointerLock: {
    persist: false,
    idempotent: true,
    alwaysDisallow: false,
    requiresRefresh: false,
    experimental: false
  },
  fullscreen: {
    persist: true,
    idempotent: false,
    alwaysAllow: false,
    alwaysDisallow: false,
    requiresRefresh: false,
    experimental: false
  },
  download: {
    persist: false,
    idempotent: false,
    alwaysDisallow: false,
    requiresRefresh: false,
    experimental: false
  },
  openExternal: {
    persist: false,
    idempotent: false,
    alwaysDisallow: false,
    requiresRefresh: false,
    experimental: false
  },
  experimentalLibrary: {
    persist: true,
    idempotent: true,
    alwaysDisallow: false,
    requiresRefresh: false,
    experimental: true
  },
  experimentalLibraryRequestAdd: {
    persist: false,
    idempotent: false,
    alwaysDisallow: false,
    requiresRefresh: false,
    experimental: true
  },
  experimentalLibraryRequestRemove: {
    persist: false,
    idempotent: false,
    alwaysDisallow: false,
    requiresRefresh: false,
    experimental: true
  },
  experimentalGlobalFetch: {
    persist: true,
    idempotent: true,
    alwaysDisallow: false,
    requiresRefresh: false,
    experimental: true
  },
  experimentalDatPeers: {
    persist: true,
    idempotent: true,
    alwaysAllow: true,
    alwaysDisallow: false,
    requiresRefresh: false,
    experimental: true
  },
  experimentalCapturePage: {
    persist: false,
    idempotent: false,
    alwaysDisallow: false,
    requiresRefresh: false,
    experimental: true
  }
};

const dat$1 = beakerCore.dat;
const sitedata = beakerCore.dbs.sitedata;
// globals
// =

var idCounter = 0;
var activeRequests = [];

// exported api
// =

function setup$12 () {
  // wire up handlers
  electron.session.defaultSession.setPermissionRequestHandler(onPermissionRequestHandler);
}

function requestPermission (permission, webContents$$1, opts) {
  return new Promise((resolve, reject) => onPermissionRequestHandler(webContents$$1, permission, resolve, opts))
}

function grantPermission (permission, webContents$$1) {
  var siteURL = (typeof webContents$$1 === 'string') ? webContents$$1 : webContents$$1.getURL();

  // update the DB
  const PERM = PERMS[_beaker_core_lib_strings.getPermId(permission)];
  if (PERM && PERM.persist) {
    sitedata.setPermission(siteURL, permission, 1);
  }
  return Promise.resolve()
}

function revokePermission (permission, webContents$$1) {
  var siteURL = (typeof webContents$$1 === 'string') ? webContents$$1 : webContents$$1.getURL();

  // update the DB
  const PERM = PERMS[_beaker_core_lib_strings.getPermId(permission)];
  if (PERM && PERM.persist) {
    sitedata.clearPermission(siteURL, permission);
  }
  return Promise.resolve()
}

function queryPermission (permission, webContents$$1) {
  return sitedata.getPermission(webContents$$1.getURL(), permission)
}

function denyAllRequests (win) {
  // remove all requests in the window, denying as we go
  activeRequests = activeRequests.filter(req => {
    if (req.win === win) {
      req.cb(false);
      return false
    }
    return true
  });
}

async function checkLabsPerm ({perm, labApi, apiDocsUrl, sender}) {
  var urlp = parseDatURL(sender.getURL());
  if (urlp.protocol === 'beaker:') return true
  if (urlp.protocol === 'dat:') {
    // resolve name
    let key = await dat$1.dns.resolveName(urlp.hostname);

    // check dat.json for opt-in
    let isOptedIn = false;
    let archive = dat$1.library.getArchive(key);
    if (archive) {
      let {checkoutFS} = dat$1.library.getArchiveCheckout(archive, urlp.version);
      let manifest = await pda.readManifest(checkoutFS).catch(_ => {});
      let apis = _get(manifest, 'experimental.apis');
      if (apis && Array.isArray(apis)) {
        isOptedIn = apis.includes(labApi);
      }
    }
    if (!isOptedIn) {
      throw new beakerErrorConstants.PermissionsError(`You must include "${labApi}" in your dat.json experimental.apis list. See ${apiDocsUrl} for more information.`)
    }

    // ask user
    let allowed = await requestPermission(perm, sender);
    if (!allowed) throw new beakerErrorConstants.UserDeniedError()
    return true
  }
  throw new beakerErrorConstants.PermissionsError()
}

// event handlers
// =

async function onPermissionRequestHandler (webContents$$1, permission, cb, opts) {
  const url$$1 = webContents$$1.getURL();

  // always allow beaker:// origins
  if (url$$1.startsWith('beaker://')) {
    return cb(true)
  }

  // look up the containing window
  var {win, view} = getContaining(webContents$$1);
  if (!win || !view) {
    console.error('Warning: failed to find containing window of permission request, ' + permission);
    return cb(false)
  }

  // check if the perm is auto-allowed or auto-disallowed
  const PERM = PERMS[_beaker_core_lib_strings.getPermId(permission)];
  if (!PERM) return cb(false)
  if (PERM && PERM.alwaysAllow) return cb(true)
  if (PERM && PERM.alwaysDisallow) return cb(false)

  // check the sitedatadb
  var res = await sitedata.getPermission(url$$1, permission).catch(err => undefined);
  if (res === 1) return cb(true)
  if (res === 0) return cb(false)

  // if we're already tracking this kind of permission request, and the perm is idempotent, then bundle them
  var req = PERM.idempotent ? activeRequests.find(req => req.view === view && req.permission === permission) : false;
  if (req) {
    var oldCb = req.cb;
    req.cb = decision => { oldCb(decision); cb(decision); };
    return
  } else {
    // track the new cb
    req = { id: ++idCounter, view, win, url: url$$1, permission, cb };
    activeRequests.push(req);
  }

  // run the UI flow
  var decision = await create$1(win, view, {permission, url: url$$1, opts});

  // persist decisions
  if (PERM && PERM.persist) {
    if (PERM.persist === 'allow' && !decision) {
      // only persist allows
      await sitedata.clearPermission(req.url, req.permission);
    } else {
      // persist all decisions
      await sitedata.setPermission(req.url, req.permission, decision);
    }
  }

  // untrack
  activeRequests.splice(activeRequests.indexOf(req), 1);

  // hand down the decision
  req.cb(decision);
}

function getContaining (webContents$$1) {
  var view = electron.BrowserView.fromWebContents(webContents$$1);
  if (view) {
    var win = findContainingWindow(view);
    return {win, view}
  }
  return {}
}


var permissions = Object.freeze({
	setup: setup$12,
	requestPermission: requestPermission,
	grantPermission: grantPermission,
	revokePermission: revokePermission,
	queryPermission: queryPermission,
	denyAllRequests: denyAllRequests,
	checkLabsPerm: checkLabsPerm
});

// globals
// =

// downloads list
// - shared across all windows
var downloads = [];

// used for rpc
var downloadsEvents = new Events();

// exported api
// =

function setup$11 () {
}

const WEBAPI$1 = { createEventsStream: createEventsStream$1, getDownloads, pause, resume, cancel, remove: remove$1, open: open$1, showInFolder };

function registerListener (win, opts = {}) {
  const listener = async (e, item, wc) => {
    // dont touch if already being handled
    // - if `opts.saveAs` is being used, there may be multiple active event handlers
    if (item.isHandled) { return }

    // track as an active download
    item.id = ('' + Date.now()) + ('' + Math.random());
    if (opts.saveAs) item.setSavePath(opts.saveAs);
    item.isHandled = true;
    item.downloadSpeed = speedometer();

    downloads.push(item);

    // This is to prevent the browser-dropdown-menu from opening
    // For now it is being used when downloading `.html` pages
    if (!opts.suppressNewDownloadEvent) {
      downloadsEvents.emit('new-download', toJSON(item));
      openOrFocusDownloadsPage(win);
    }

    var lastBytes = 0;
    item.on('updated', () => {
      // set name if not already done
      if (!item.name) {
        item.name = path__default.basename(item.getSavePath());
      }

      var sumProgress = {
        receivedBytes: getSumReceivedBytes(),
        totalBytes: getSumTotalBytes()
      };

      // track rate of download
      item.downloadSpeed(item.getReceivedBytes() - lastBytes);
      lastBytes = item.getReceivedBytes();

      // emit
      downloadsEvents.emit('updated', toJSON(item));
      downloadsEvents.emit('sum-progress', sumProgress);
      win.setProgressBar(sumProgress.receivedBytes / sumProgress.totalBytes);
    });

    item.on('done', (e, state) => {
      // inform users of error conditions
      var overrides = false;
      if (state === 'interrupted') {
        // this can sometimes happen because the link is a data: URI
        // in that case, we can manually parse and save it
        if (item.getURL().startsWith('data:')) {
          let parsed = parseDataURL(item.getURL());
          if (parsed) {
            fs__default.writeFileSync(item.getSavePath(), parsed.body);
            overrides = {
              state: 'completed',
              receivedBytes: parsed.body.length,
              totalBytes: parsed.body.length
            };
          }
        }
        if (!overrides) {
          electron.dialog.showErrorBox('Download error', `The download of ${item.getFilename()} was interrupted`);
        }
      }

      downloadsEvents.emit('done', toJSON(item, overrides));

      // replace entry with a clone that captures the final state
      downloads.splice(downloads.indexOf(item), 1, capture(item, overrides));

      // reset progress bar when done
      if (isNoActiveDownloads() && !win.isDestroyed()) {
        win.setProgressBar(-1);
      }

      if (state === 'completed') {
        // flash the dock on osx
        if (process.platform === 'darwin') {
          electron.app.dock.downloadFinished(item.getSavePath());
        }
      }

      // optional, for one-time downloads
      if (opts.unregisterWhenDone) {
        wc.session.removeListener('will-download', listener);
      }
    });
  };

  win.webContents.session.prependListener('will-download', listener);
  win.on('close', () => win.webContents.session.removeListener('will-download', listener));
}

function download (win, wc, url$$1, opts) {
  // register for onetime use of the download system
  opts = Object.assign({}, opts, {unregisterWhenDone: true, trusted: true});
  registerListener(win, opts);
  wc.downloadURL(url$$1);
}

// rpc api
// =

function createEventsStream$1 () {
  return emitStream(downloadsEvents)
}

function getDownloads () {
  return Promise.resolve(downloads.map(toJSON))
}

function pause (id) {
  var download = downloads.find(d => d.id == id);
  if (download) { download.pause(); }
  return Promise.resolve()
}

function resume (id) {
  var download = downloads.find(d => d.id == id);
  if (download) { download.resume(); }
  return Promise.resolve()
}

function cancel (id) {
  var download = downloads.find(d => d.id == id);
  if (download) { download.cancel(); }
  return Promise.resolve()
}

function remove$1 (id) {
  var download = downloads.find(d => d.id == id);
  if (download && download.getState() != 'progressing') { downloads.splice(downloads.indexOf(download), 1); }
  return Promise.resolve()
}

function open$1 (id) {
  return new Promise((resolve, reject) => {
    // find the download
    var download = downloads.find(d => d.id == id);
    if (!download || download.state != 'completed') { return reject() }

    // make sure the file is still there
    fs__default.stat(download.getSavePath(), err => {
      if (err) { return reject() }

      // open
      electron.shell.openItem(download.getSavePath());
      resolve();
    });
  })
}

function showInFolder (id) {
  return new Promise((resolve, reject) => {
    // find the download
    var download = downloads.find(d => d.id == id);
    if (!download || download.state != 'completed') { return reject() }

    // make sure the file is still there
    fs__default.stat(download.getSavePath(), err => {
      if (err) { return reject() }

      // open
      electron.shell.showItemInFolder(download.getSavePath());
      resolve();
    });
  })
}

// internal helpers
// =

// reduce down to attributes
function toJSON (item, overrides) {
  return {
    id: item.id,
    name: item.name,
    url: item.getURL(),
    state: overrides ? overrides.state : item.getState(),
    isPaused: item.isPaused(),
    receivedBytes: overrides ? overrides.receivedBytes : item.getReceivedBytes(),
    totalBytes: overrides ? overrides.totalBytes : item.getTotalBytes(),
    downloadSpeed: item.downloadSpeed()
  }
}

// create a capture of the final state of an item
function capture (item, overrides) {
  var savePath = item.getSavePath();
  var dlspeed = item.download;
  item = toJSON(item, overrides);
  item.getURL = () => item.url;
  item.getState = () => overrides === true ? 'completed' : item.state;
  item.isPaused = () => false;
  item.getReceivedBytes = () => overrides ? overrides.receivedBytes : item.receivedBytes;
  item.getTotalBytes = () => overrides ? overrides.totalBytes : item.totalBytes;
  item.getSavePath = () => savePath;
  item.downloadSpeed = () => dlspeed;
  return item
}

// sum of received bytes
function getSumReceivedBytes () {
  return getActiveDownloads().reduce((acc, item) => acc + item.getReceivedBytes(), 0)
}

// sum of total bytes
function getSumTotalBytes () {
  return getActiveDownloads().reduce((acc, item) => acc + item.getTotalBytes(), 0)
}

function getActiveDownloads () {
  return downloads.filter(d => d.getState() == 'progressing')
}

// all downloads done?
function isNoActiveDownloads () {
  return getActiveDownloads().length === 0
}

// exported APIs
// =

function setup$10 () {
  setApplicationMenu({ noWindows: true });

  // watch for changes to the currently active window
  electron.app.on('browser-window-focus', async (e, win) => {
    try {
      // fetch the current url
      const url$$1 = getActive(win).url;

      // rebuild as needed
      if (requiresRebuild(url$$1)) {
        setApplicationMenu({url: url$$1});
      }
    } catch (e) {
      // `pages` not set yet
    }
  });

  // watch for all windows to be closed
  electron.app.on('custom-window-all-closed', () => {
    setApplicationMenu({ noWindows: true });
  });

  // watch for any window to be opened
  electron.app.on('browser-window-created', () => {
    setApplicationMenu();
  });
}

function onSetCurrentLocation (win, url$$1) {
  // check if this is the currently focused window
  if (!url$$1 || win !== electron.BrowserWindow.getFocusedWindow()) {
    return
  }

  // rebuild as needed
  if (requiresRebuild(url$$1)) {
    setApplicationMenu({url: url$$1});
  }
}

function setApplicationMenu (opts = {}) {
  electron.Menu.setApplicationMenu(electron.Menu.buildFromTemplate(buildWindowMenu(opts)));
}

function buildWindowMenu (opts = {}) {
  const isDat = opts.url && opts.url.startsWith('dat://');
  const noWindows = opts.noWindows === true;

  var darwinMenu = {
    label: 'Beaker',
    submenu: [
      {
        label: 'Preferences',
        accelerator: 'Command+,',
        click (item, win) {
          if (win) create$$1(win, 'beaker://settings', {setActive: true});
          else createShellWindow({ pages: ['beaker://settings'] });
        }
      },
      { type: 'separator' },
      { label: 'Services', role: 'services', submenu: [] },
      { type: 'separator' },
      { label: 'Hide Beaker', accelerator: 'Command+H', role: 'hide' },
      { label: 'Hide Others', accelerator: 'Command+Alt+H', role: 'hideothers' },
      { label: 'Show All', role: 'unhide' },
      { type: 'separator' },
      { label: 'Quit', accelerator: 'Command+Q', click () { electron.app.quit(); }, reserved: true }
    ]
  };

  var fileMenu = {
    label: 'File',
    submenu: [
      {
        label: 'New Tab',
        accelerator: 'CmdOrCtrl+T',
        click: function (item, win) {
          if (win) {
            create$$1(win, undefined, {setActive: true, focusLocationBar: true});
          } else {
            createShellWindow();
          }
        },
        reserved: true
      },
      {
        label: 'New Window',
        accelerator: 'CmdOrCtrl+N',
        click: function () { createShellWindow(); },
        reserved: true
      },
      {
        label: 'Reopen Closed Tab',
        accelerator: 'CmdOrCtrl+Shift+T',
        click: function (item, win) {
          createWindowIfNone(win, (win) => {
            reopenLastRemoved(win);
          });
        },
        reserved: true
      },
      {
        label: 'Open File',
        accelerator: 'CmdOrCtrl+O',
        click: function (item, win) {
          createWindowIfNone(win, async (win) => {
            var {filePaths} = await electron.dialog.showOpenDialog({ title: 'Open file...', properties: ['openFile', 'createDirectory'] });
            if (filePaths && filePaths[0]) { create$$1(win, 'file://' + filePaths[0], {setActive: true}); }
          });
        }
      },
      {
        label: 'Open Location',
        accelerator: 'CmdOrCtrl+L',
        click: function (item, win) {
          createWindowIfNone(win, (win) => {
            win.webContents.send('command', 'focus-location');
          });
        }
      },
      { type: 'separator' },
      {
        label: 'Save Page As...',
        enabled: !noWindows,
        accelerator: 'CmdOrCtrl+S',
        click: async (item, win) => {
          var view = getActive(win);
          if (!view) return
          const url$$1 = view.url;
          const title = view.title;
          var {filePath} = await electron.dialog.showSaveDialog({ title: `Save ${title} as...`, defaultPath: electron.app.getPath('downloads') });
          if (filePath) download(win, win.webContents, url$$1, { saveAs: filePath, suppressNewDownloadEvent: true });
        }
      },
      {
        label: 'Print...',
        enabled: !noWindows,
        accelerator: 'CmdOrCtrl+P',
        click: (item, win) => {
          var view = getActive(win);
          if (!view) return
          view.webContents.print();
        }
      },
      { type: 'separator' },
      {
        label: 'Close Window',
        enabled: !noWindows,
        accelerator: 'CmdOrCtrl+Shift+W',
        click: function (item, win) {
          if (win) win.close();
        },
        reserved: true
      },
      {
        label: 'Close Tab',
        enabled: !noWindows,
        accelerator: 'CmdOrCtrl+W',
        click: function (item, win) {
          if (win) {
            // a regular browser window
            let active = getActive(win);
            if (active) remove(win, active);
          } else {
            // devtools
            let wc = getFocusedDevToolsHost();
            if (wc) {
              wc.closeDevTools();
            }
          }
        },
        reserved: true
      }
    ]
  };

  var editMenu = {
    label: 'Edit',
    submenu: [
      { label: 'Undo', enabled: !noWindows, accelerator: 'CmdOrCtrl+Z', selector: 'undo:', reserved: true },
      { label: 'Redo', enabled: !noWindows, accelerator: 'Shift+CmdOrCtrl+Z', selector: 'redo:', reserved: true },
      { type: 'separator' },
      { label: 'Cut', enabled: !noWindows, accelerator: 'CmdOrCtrl+X', selector: 'cut:', reserved: true },
      { label: 'Copy', enabled: !noWindows, accelerator: 'CmdOrCtrl+C', selector: 'copy:', reserved: true },
      { label: 'Paste', enabled: !noWindows, accelerator: 'CmdOrCtrl+V', selector: 'paste:', reserved: true },
      { label: 'Select All', enabled: !noWindows, accelerator: 'CmdOrCtrl+A', selector: 'selectAll:' },
      {
        label: 'Find in Page',
        enabled: !noWindows,
        accelerator: 'CmdOrCtrl+F',
        click: function (item, win) {
          var view = getActive(win);
          if (view) view.showInpageFind();
        }
      },
      {
        label: 'Find Next',
        enabled: !noWindows,
        accelerator: 'CmdOrCtrl+G',
        click: function (item, win) {
          var view = getActive(win);
          if (view) view.moveInpageFind(1);
        }
      },
      {
        label: 'Find Previous',
        enabled: !noWindows,
        accelerator: 'Shift+CmdOrCtrl+G',
        click: function (item, win) {
          var view = getActive(win);
          if (view) view.moveInpageFind(-1);
        }
      }
    ]
  };

  var viewMenu = {
    label: 'View',
    submenu: [{
      label: 'Reload',
      enabled: !noWindows,
      accelerator: 'CmdOrCtrl+R',
      click: function (item, win) {
        if (win) {
          let active = getActive(win);
          if (active) {
            active.webContents.reload();
          }
        }
      },
      reserved: true
    },
      {
        label: 'Hard Reload (Clear Cache)',
        accelerator: 'CmdOrCtrl+Shift+R',
        click: function (item, win) {
          // HACK
          // this is *super* lazy but it works
          // clear all dat-dns cache on hard reload, to make sure the next
          // load is fresh
          // -prf
          beakerCore.dat.dns.flushCache();

          if (win) {
            let active = getActive(win);
            if (active) {
              active.webContents.reloadIgnoringCache();
            }
          }
        },
        reserved: true
      },
    { type: 'separator' },
      {
        label: 'Zoom In',
        enabled: !noWindows,
        accelerator: 'CmdOrCtrl+Plus',
        reserved: true,
        click: function (item, win) {
          if (win) {
            zoomIn(getActive(win));
          }
        }
      },
      {
        label: 'Zoom Out',
        enabled: !noWindows,
        accelerator: 'CmdOrCtrl+-',
        reserved: true,
        click: function (item, win) {
          if (win) {
            zoomOut(getActive(win));
          }
        }
      },
      {
        label: 'Actual Size',
        enabled: !noWindows,
        accelerator: 'CmdOrCtrl+0',
        click: function (item, win) {
          if (win) {
            zoomReset(getActive(win));
          }
        }
      },
    { type: 'separator' },
      {
        type: 'submenu',
        label: 'Advanced Tools',
        submenu: [{
          label: 'Reload Shell-Window',
          enabled: !noWindows,
          accelerator: 'CmdOrCtrl+alt+shift+R',
          click: function () {
            electron.BrowserWindow.getFocusedWindow().webContents.reloadIgnoringCache();
          }
        }, {
          label: 'Toggle Shell-Window DevTools',
          enabled: !noWindows,
          accelerator: 'CmdOrCtrl+alt+shift+I',
          click: function () {
            electron.BrowserWindow.getFocusedWindow().toggleDevTools();
          }
        },
      { type: 'separator' },
          {
            label: 'Open Archives Debug Page',
            enabled: !noWindows,
            click: function (item, win) {
              if (win) create$$1(win, 'beaker://internal-archives/', {setActive: true});
            }
          }, {
            label: 'Open Dat-DNS Cache Page',
            enabled: !noWindows,
            click: function (item, win) {
              if (win) create$$1(win, 'beaker://dat-dns-cache/', {setActive: true});
            }
          }, {
            label: 'Open Debug Log Page',
            enabled: !noWindows,
            click: function (item, win) {
              if (win) create$$1(win, 'beaker://debug-log/', {setActive: true});
            }
          }]
      },
      {
        label: 'Toggle DevTools',
        enabled: !noWindows,
        accelerator: (process.platform === 'darwin') ? 'Alt+CmdOrCtrl+I' : 'Shift+CmdOrCtrl+I',
        click: function (item, win) {
          if (win) {
            let active = getActive(win);
            if (active) active.webContents.toggleDevTools();
          }
        },
        reserved: true
      },
      {
        label: 'Toggle Javascript Console',
        enabled: !noWindows,
        accelerator: (process.platform === 'darwin') ? 'Alt+CmdOrCtrl+J' : 'Shift+CmdOrCtrl+J',
        click: function (item, win) {
          if (win) {
            let active = getActive(win);
            if (active) {
              const onOpened = () => {
                const dtwc = active.webTools.devToolsWebContents;
                if (dtwc) dtwc.executeJavaScript('DevToolsAPI.showPanel("console")');
              };
              if (!active.webContents.isDevToolsOpened()) {
                active.webContents.once('devtools-opened', onOpened);
                active.webContents.toggleDevTools();
              } else {
                onOpened();
              }
            }
          }
        },
        reserved: true
      },
      {
        label: 'Toggle Live Reloading',
        enabled: !!isDat,
        click: function (item, win) {
          if (win) {
            let active = getActive(win);
            if (active) active.toggleLiveReloading();
          }
        }
      },
      { type: 'separator' },
      {
        label: 'Full Screen',
        enabled: !noWindows,
        accelerator: (process.platform === 'darwin') ? 'Ctrl+Cmd+F' : 'F11',
        role: 'toggleFullScreen'
      }
    ]
  };

  var showHistoryAccelerator = 'Ctrl+h';

  if (process.platform === 'darwin') {
    showHistoryAccelerator = 'Cmd+y';
  }

  var historyMenu = {
    label: 'History',
    role: 'history',
    submenu: [
      {
        label: 'Back',
        enabled: !noWindows,
        accelerator: 'CmdOrCtrl+Left',
        click: function (item, win) {
          if (win) {
            let active = getActive(win);
            if (active) active.webContents.goBack();
          }
        }
      },
      {
        label: 'Forward',
        enabled: !noWindows,
        accelerator: 'CmdOrCtrl+Right',
        click: function (item, win) {
          if (win) {
            let active = getActive(win);
            if (active) active.webContents.goForward();
          }
        }
      },
      {
        label: 'Show Full History',
        accelerator: showHistoryAccelerator,
        click: function (item, win) {
          if (win) create$$1(win, 'beaker://history', {setActive: true});
          else createShellWindow({ pages: ['beaker://history'] });
        }
      },
      { type: 'separator' },
      {
        label: 'Bookmark this Page',
        enabled: !noWindows,
        accelerator: 'CmdOrCtrl+D',
        click: function (item, win) {
          if (win) win.webContents.send('command', 'create-bookmark');
        }
      }
    ]
  };

  var windowMenu = {
    label: 'Window',
    role: 'window',
    submenu: [
      {
        label: 'Minimize',
        accelerator: 'CmdOrCtrl+M',
        role: 'minimize'
      },
      {
        label: 'Next Tab',
        enabled: !noWindows,
        accelerator: 'CmdOrCtrl+}',
        click: function (item, win) {
          if (win) changeActiveBy(win, 1);
        }
      },
      {
        label: 'Previous Tab',
        enabled: !noWindows,
        accelerator: 'CmdOrCtrl+{',
        click: function (item, win) {
          if (win) changeActiveBy(win, -1);
        }
      }
    ]
  };
  if (process.platform == 'darwin') {
    windowMenu.submenu.push({
      type: 'separator'
    });
    windowMenu.submenu.push({
      label: 'Bring All to Front',
      role: 'front'
    });
  }

  var helpMenu = {
    label: 'Help',
    role: 'help',
    submenu: [
      {
        label: 'Help',
        accelerator: 'F1',
        click: function (item, win) {
          if (win) create$$1(win, 'https://beakerbrowser.com/docs/', {setActive: true});
        }
      },
      {
        label: 'Report Bug',
        click: function (item, win) {
          if (win) create$$1(win, 'https://github.com/beakerbrowser/beaker/issues', {setActive: true});
        }
      },
      {
        label: 'Mailing List',
        click: function (item, win) {
          if (win) create$$1(win, 'https://groups.google.com/forum/#!forum/beaker-browser', {setActive: true});
        }
      }
    ]
  };
  if (process.platform !== 'darwin') {
    helpMenu.submenu.push({ type: 'separator' });
    helpMenu.submenu.push({
      label: 'About',
      role: 'about',
      click: function (item, win) {
        if (win) create$$1(win, 'beaker://settings', {setActive: true});
      }
    });
  }

  // assemble final menu
  var menus = [fileMenu, editMenu, viewMenu, historyMenu, windowMenu, helpMenu];
  if (process.platform === 'darwin') menus.unshift(darwinMenu);
  return menus
}

// internal helpers
// =

var lastURLProtocol = false;
function requiresRebuild (url$$1) {
  const urlProtocol = url$$1 ? url$$1.split(':')[0] : false;
  // check if this is a change of protocol
  const b = (lastURLProtocol !== urlProtocol);
  lastURLProtocol = urlProtocol;
  return b
}

function createWindowIfNone (win, onShow) {
  if (win) return onShow(win)
  win = createShellWindow();
  win.once('show', onShow.bind(null, win));
}

const isDatHashRegex = /^[a-z0-9]{64}/i;
const isIPAddressRegex = /^\d{1,3}\.\d{1,3}\.\d{1,3}\.\d{1,3}/;

// helper to determine what the user may be inputting into the locaiton bar
function examineLocationInput (v) {
  // does the value look like a url?
  var isProbablyUrl = (!v.includes(' ') && (
    /\.[A-z]/.test(v) ||
    isIPAddressRegex.test(v) ||
    isDatHashRegex.test(v) ||
    v.startsWith('localhost') ||
    v.includes('://') ||
    v.startsWith('beaker:') ||
    v.startsWith('data:')
  ));
  var vWithProtocol = v;
  var isGuessingTheScheme = false;
  if (isProbablyUrl && !v.includes('://') && !(v.startsWith('beaker:') || v.startsWith('data:'))) {
    if (isDatHashRegex.test(v)) {
      vWithProtocol = 'dat://' + v;
    } else if (v.startsWith('localhost') || isIPAddressRegex.test(v)) {
      vWithProtocol = 'http://' + v;
    } else {
      vWithProtocol = 'https://' + v;
      isGuessingTheScheme = true; // note that we're guessing so that, if this fails, we can try http://
    }
  }
  var vSearch = 'https://duckduckgo.com/?q=' + v.split(' ').map(encodeURIComponent).join('+');
  return {vWithProtocol, vSearch, isProbablyUrl, isGuessingTheScheme}
}

const settingsDb$2 = beakerCore.dbs.settings;
const historyDb = beakerCore.dbs.history;
const bookmarksDb = beakerCore.dbs.bookmarks;

const ERR_ABORTED = -3;
const ERR_CONNECTION_REFUSED = -102;
const ERR_INSECURE_RESPONSE = -501;
const TLS_ERROR_CODES = Object.values({
  ERR_NO_SSL_VERSIONS_ENABLED: -112,
  ERR_SSL_VERSION_OR_CIPHER_MISMATCH: -113,
  ERR_SSL_RENEGOTIATION_REQUESTED: -114,
  ERR_PROXY_AUTH_UNSUPPORTED: -115,
  ERR_CERT_ERROR_IN_SSL_RENEGOTIATION: -116,
  ERR_BAD_SSL_CLIENT_AUTH_CERT: -117,
  ERR_SSL_NO_RENEGOTIATION: -123,
  ERR_SSL_WEAK_SERVER_EPHEMERAL_DH_KEY: -129,
  ERR_PROXY_CERTIFICATE_INVALID: -136,
  ERR_SSL_HANDSHAKE_NOT_COMPLETED: -148,
  ERR_SSL_BAD_PEER_PUBLIC_KEY: -149,
  ERR_SSL_PINNED_KEY_NOT_IN_CERT_CHAIN: -150,
  ERR_CLIENT_AUTH_CERT_TYPE_UNSUPPORTED: -151,
  ERR_SSL_DECRYPT_ERROR_ALERT: -153,
  ERR_SSL_SERVER_CERT_CHANGED: -156,
  ERR_SSL_UNRECOGNIZED_NAME_ALERT: -159,
  ERR_SSL_SERVER_CERT_BAD_FORMAT: -167,
  ERR_CT_STH_PARSING_FAILED: -168,
  ERR_CT_STH_INCOMPLETE: -169,
  ERR_CT_CONSISTENCY_PROOF_PARSING_FAILED: -171,
  ERR_SSL_OBSOLETE_CIPHER: -172,
  ERR_SSL_VERSION_INTERFERENCE: -175,
  ERR_EARLY_DATA_REJECTED: -178,
  ERR_WRONG_VERSION_ON_EARLY_DATA: -179,
  ERR_TLS13_DOWNGRADE_DETECTED: -180
});
const IS_CODE_INSECURE_RESPONSE = x => x === ERR_CONNECTION_REFUSED || x === ERR_INSECURE_RESPONSE || (x <= -200 && x > -300) || TLS_ERROR_CODES.includes(x);

const Y_POSITION = 78;
const DEFAULT_URL = 'beaker://start';
const TRIGGER_LIVE_RELOAD_DEBOUNCE = 500; // throttle live-reload triggers by this amount

// the variables which are automatically sent to the shell-window for rendering
const STATE_VARS = [
  'url',
  'title',
  'peers',
  'favicons',
  'zoom',
  'loadError',
  'isActive',
  'isPinned',
  'isBookmarked',
  'isLoading',
  'isReceivingAssets',
  'canGoBack',
  'canGoForward',
  'isAudioMuted',
  'isCurrentlyAudible',
  'isInpageFindActive',
  'currentInpageFindString',
  'currentInpageFindResults',
  'availableAlternative',
  'donateLinkHref',
  'localPath',
  'isLiveReloading'
];

// globals
// =

var activeViews = {}; // map of {[win.id]: Array<View>}
var closedURLs = {}; // map of {[win.id]: Array<string>}
var windowEvents = {}; // mapof {[win.id]: Events}
var noRedirectHostnames = new Set(); // set of hostnames which have dat-redirection disabled
var nextViewIsScriptCloseable = false; // will the next view created be "script closable"?

// classes
// =

class View {
  constructor (win, opts = {isPinned: false}) {
    this.browserWindow = win;
    this.browserView = new electron.BrowserView({
      webPreferences: {
        preload: path__default.join(__dirname, 'webview-preload.build.js'),
        contextIsolation: true,
        webviewTag: false,
        sandbox: true,
        defaultEncoding: 'utf-8',
        nativeWindowOpen: true,
        nodeIntegration: false,
        scrollBounce: true,
        navigateOnDragDrop: true
      }
    });
    this.browserView.setBackgroundColor('#fff');

    // webview state
    this.loadingURL = null; // URL being loaded, if any
    this.isLoading = false; // is the tab loading?
    this.isReceivingAssets = false; // has the webview started receiving assets in the current load-cycle?
    this.favicons = null; // array of favicon URLs
    this.zoom = 0; // what's the current zoom level?
    this.loadError = null; // page error state, if any

    // browser state
    this.isActive = false; // is this the active page in the window?
    this.isPinned = Boolean(opts.isPinned); // is this page pinned?
    this.liveReloadEvents = null; // live-reload event stream
    this.isInpageFindActive = false; // is the inpage-finder UI active?
    this.currentInpageFindString = undefined; // what's the current inpage-finder query string?
    this.currentInpageFindResults = undefined; // what's the current inpage-finder query results?
    this.isScriptClosable = takeIsScriptClosable(); // can this view be closed by `window.close` ?

    // helper state
    this.peers = 0; // how many peers does the site have?
    this.isBookmarked = false; // is the active page bookmarked?
    this.datInfo = null; // metadata about the site if viewing a dat
    this.donateLinkHref = null; // the URL of the donate site, if set by the dat.json
    this.localPath = null; // the path of the local sync directory, if set
    this.availableAlternative = ''; // tracks if there's alternative protocol available for the site
    this.wasDatTimeout = false; // did the last navigation result in a timed-out dat?

    // wire up events
    this.webContents.on('did-start-loading', this.onDidStartLoading.bind(this));
    this.webContents.on('did-start-navigation', this.onDidStartNavigation.bind(this));
    this.webContents.on('did-navigate', this.onDidNavigate.bind(this));
    this.webContents.on('did-navigate-in-page', this.onDidNavigateInPage.bind(this));
    this.webContents.on('did-stop-loading', this.onDidStopLoading.bind(this));
    this.webContents.on('did-fail-load', this.onDidFailLoad.bind(this));
    this.webContents.on('update-target-url', this.onUpdateTargetUrl.bind(this));
    this.webContents.on('page-title-updated', this.onPageTitleUpdated.bind(this)); // NOTE page-title-updated isn't documented on webContents but it is supported
    this.webContents.on('page-favicon-updated', this.onPageFaviconUpdated.bind(this));
    this.webContents.on('new-window', this.onNewWindow.bind(this));
    this.webContents.on('media-started-playing', this.onMediaChange.bind(this));
    this.webContents.on('media-paused', this.onMediaChange.bind(this));
    this.webContents.on('found-in-page', this.onFoundInPage.bind(this));

    // security - deny these events
    const deny = e => e.preventDefault();
    this.webContents.on('remote-require', deny);
    this.webContents.on('remote-get-global', deny);
    this.webContents.on('remote-get-builtin', deny);
    this.webContents.on('remote-get-current-window', deny);
    this.webContents.on('remote-get-current-web-contents', deny);
    this.webContents.on('remote-get-guest-web-contents', deny);
  }

  get webContents () {
    return this.browserView.webContents
  }

  get url () {
    return this.webContents.getURL()
  }

  get origin () {
    return toOrigin(this.url)
  }

  get title () {
    var title = this.webContents.getTitle();
    if (this.datInfo && this.datInfo.title && (!title || title.startsWith(this.origin))) {
      // fallback to the dat.json title field if the page doesnt provide a title
      title = this.datInfo.title;
    }
    return title
  }

  get canGoBack () {
    return this.webContents.canGoBack()
  }

  get canGoForward () {
    return this.webContents.canGoForward()
  }

  get isAudioMuted () {
    return this.webContents.isAudioMuted()
  }

  get isCurrentlyAudible () {
    return this.webContents.isCurrentlyAudible()
  }

  get isLiveReloading () {
    return !!this.liveReloadEvents
  }

  get state () {
    var state = _pick(this, STATE_VARS);
    if (this.loadingURL) state.url = this.loadingURL;
    return state
  }

  // management
  // =

  loadURL (url$$1) {
    this.browserView.webContents.loadURL(url$$1);
  }

  resize () {
    const win = this.browserWindow;
    var {width, height} = win.getContentBounds();
    this.browserView.setBounds({x: 0, y: Y_POSITION, width, height: height - Y_POSITION});
    this.browserView.setAutoResize({width: true, height: true});
  }

  activate () {
    this.isActive = true;

    const win = this.browserWindow;
    win.setBrowserView(this.browserView);
    show$3(this.browserView);
    show$4(this.browserView);

    this.resize();
    this.webContents.focus();
  }

  deactivate (dontNullTheView = false) {
    if (!dontNullTheView && this.isActive) {
      this.browserWindow.setBrowserView(null);
    }

    if (this.isActive) {
      hide(this.browserWindow); // this will close the location menu if it's open
    }
    hide$3(this.browserView);
    hide$4(this.browserView);
    this.isActive = false;
  }

  destroy () {
    this.deactivate();
    close(this.browserView);
    close$1(this.browserView);
    this.browserView.destroy();
  }

  async updateHistory () {
    var url$$1 = this.url;
    var title = this.title;

    if (!/^beaker:\/\/(start|history)/i.test(url$$1)) {
      historyDb.addVisit(0, {url: url$$1, title});
      if (this.isPinned) {
        savePins(this.browserWindow);
      }
    }
  }

  toggleMuted () {
    this.webContents.setAudioMuted(!this.isAudioMuted);
    this.emitUpdateState();
  }

  // inpage finder
  // =

  showInpageFind () {
    if (this.isInpageFindActive) {
      // go to next result on repeat "show" commands
      this.moveInpageFind(1);
    } else {
      this.isInpageFindActive = true;
      this.currentInpageFindResults = {activeMatchOrdinal: 0, matches: 0};
      this.emitUpdateState();
    }
    this.browserWindow.webContents.focus();
  }

  hideInpageFind () {
    this.webContents.stopFindInPage('clearSelection');
    this.isInpageFindActive = false;
    this.currentInpageFindString = undefined;
    this.currentInpageFindResults = undefined;
    this.emitUpdateState();
  }

  setInpageFindString (str, dir) {
    this.currentInpageFindString = str;
    this.webContents.findInPage(this.currentInpageFindString, {findNext: false, forward: dir !== -1});
  }

  moveInpageFind (dir) {
    this.webContents.findInPage(this.currentInpageFindString, {findNext: false, forward: dir !== -1});
  }

  // alternative protocols
  // =

  async checkForDatAlternative (url$$1) {
    let u = (new URL(url$$1));
    // try to do a name lookup
    var siteHasDatAlternative = await beakerCore.dat.dns.resolveName(u.hostname).then(
      res => Boolean(res),
      err => false
    );
    if (siteHasDatAlternative) {
      var autoRedirectToDat = !!await beakerCore.dbs.settings.get('auto_redirect_to_dat');
      if (autoRedirectToDat && !noRedirectHostnames.has(u.hostname)) {
        // automatically redirect
        let datUrl = url$$1.replace(u.protocol, 'dat:');
        this.loadURL(datUrl);
      } else {
        // track that dat is available
        this.availableAlternative = 'dat:';
      }
    } else {
      this.availableAlternative = '';
    }
    this.emitUpdateState();
  }

  // live reloading
  // =

  toggleLiveReloading () {
    if (this.liveReloadEvents) {
      this.liveReloadEvents.close();
      this.liveReloadEvents = false;
    } else if (this.datInfo) {
      let archive = beakerCore.dat.library.getArchive(this.datInfo.key);
      if (!archive) return

      let {version} = parseDatURL(this.url);
      let {checkoutFS} = beakerCore.dat.library.getArchiveCheckout(archive, version);
      this.liveReloadEvents = checkoutFS.pda.watch();

      let event = (this.datInfo.isOwner) ? 'changed' : 'invalidated';
      const reload = _throttle(() => {
        this.browserView.webContents.reload();
      }, TRIGGER_LIVE_RELOAD_DEBOUNCE, {leading: false});
      this.liveReloadEvents.on('data', ([evt]) => {
        if (evt === event) reload();
      });
      // ^ note this throttle is run on the front edge.
      // That means snappier reloads (no delay) but possible double reloads if multiple files change
    }
    this.emitUpdateState();
  }

  stopLiveReloading () {
    if (this.liveReloadEvents) {
      this.liveReloadEvents.close();
      this.liveReloadEvents = false;
      this.emitUpdateState();
    }
  }

  // custom renderers
  // =

  async injectCustomRenderers () {
    // determine content type
    let contentType = getResourceContentType(this.url) || '';
    let isMD = contentType.startsWith('text/markdown') || contentType.startsWith('text/x-markdown');
    let isJSON = contentType.startsWith('application/json');
    let isPlainText = contentType.startsWith('text/plain');

    // markdown rendering
    // inject the renderer script if the page is markdown
    if (isMD || (isPlainText && this.url.endsWith('.md'))) {
      // hide the unformatted text and provide some basic styles
      this.webContents.insertCSS(`
        body { font-family: -apple-system, BlinkMacSystemFont, "Segoe UI", Ubuntu, Cantarell, "Oxygen Sans", "Helvetica Neue", sans-serif; }
        body > pre { display: none; }
        main { display: flex; }
        nav { max-width: 200px; padding-right: 2em; }
        nav .link { white-space: pre; overflow: hidden; text-overflow: ellipsis; margin: 0.5em 0 }
        main > div { max-width: 800px; }
        hr { border: 0; border-top: 1px solid #ccc; margin: 1em 0; }
        blockquote { margin: 0; padding: 0 1em; border-left: 1em solid #eee; }
        .anchor-link { color: #aaa; margin-left: 5px; text-decoration: none; visibility: hidden; }
        h1:hover .anchor-link, h2:hover .anchor-link, h3:hover .anchor-link, h4:hover .anchor-link, h5:hover .anchor-link { visibility: visible; }
        table { border-collapse: collapse; }
        td, th { padding: 0.5em 1em; }
        tbody tr:nth-child(odd) { background: #fafafa; }
        tbody td { border-top: 1px solid #bbb; }
        .switcher { position: absolute; top: 5px; right: 5px; font-family: Consolas, 'Lucida Console', Monaco, monospace; cursor: pointer; font-size: 13px; background: #fafafa; padding: 2px 5px; }
        main code { font-size: 1.3em; background: #fafafa; }
        main pre { background: #fafafa; padding: 1em }
      `);
      this.webContents.insertCSS(`
        .markdown { font-size: 14px; width: 100%; max-width: 750px; line-height: 22.5px; }
        .markdown a { color: #2864dc; text-decoration: none; }
        .markdown a:hover { text-decoration: underline; }
        .markdown a.anchor-link { color: #ddd; }
        .markdown h1, .markdown h2, .markdown  h3 { margin: 15px 0; font-weight: 600; }
        .markdown h1, .markdown h2 { border-bottom: 1px solid #eee; line-height: 45px; }
        .markdown h1 { font-size: 30px; }
        .markdown h2 { font-size: 24px; }
        .markdown h3 { font-size: 20px; }
        .markdown ul, .markdown ol { margin-bottom: 15px; }
        .markdown pre, .markdown code { font-family: Consolas, 'Lucida Console', Monaco, monospace; font-size: 13.5px; background: #f0f0f0; border-radius: 2px; }
        .markdown pre { padding: 15px; border: 0; overflow-x: auto; }
        .markdown code { padding: 3px 5px; }
        .markdown pre > code { display: block; }
      `);
      let mdpath = path__default.join(electron.app.getAppPath(), 'markdown-renderer.build.js');
      mdpath = mdpath.replace('app.asar', 'app.asar.unpacked'); // fetch from unpacked dir
      this.webContents.executeJavaScript(await fs.promises.readFile(mdpath, 'utf8'));
    }

    // json rendering
    // inject the json render script
    if (isJSON || (isPlainText && this.url.endsWith('.json'))) {
      this.webContents.insertCSS(`
        .hidden { display: none !important; }
        .json-formatter-row {
          font-family: Consolas, 'Lucida Console', Monaco, monospace !important;
          line-height: 1.6 !important;
          font-size: 13px;
        }
        .json-formatter-row > a > .json-formatter-preview-text {
          transition: none !important;
        }
        nav { margin-bottom: 5px; user-select: none; }
        nav > span {
          cursor: pointer;
          display: inline-block;
          font-family: Consolas, "Lucida Console", Monaco, monospace;
          cursor: pointer;
          font-size: 13px;
          background: rgb(250, 250, 250);
          padding: 3px 5px;
          margin-right: 5px;
        }
        nav > span.pressed {
          box-shadow: inset 2px 2px 2px rgba(0,0,0,.05);
          background: #ddd;
        }
      `);
      let jsonpath = path__default.join(electron.app.getAppPath(), 'json-renderer.build.js');
      jsonpath = jsonpath.replace('app.asar', 'app.asar.unpacked'); // fetch from unpacked dir
      this.webContents.executeJavaScript(await fs.promises.readFile(jsonpath, 'utf8'));
    }
  }

  // state fetching
  // =

  // helper called by UIs to pull latest state if a change event has occurred
  // eg called by the bookmark systems after the bookmark state has changed
  async refreshState () {
    await Promise.all([
      this.fetchIsBookmarked(true),
      this.fetchDatInfo(true)
    ]);
    this.emitUpdateState();
  }

  async fetchIsBookmarked (noEmit = false) {
    var bookmark = await bookmarksDb.getBookmark(0, normalizeURL(this.url, {
      stripFragment: false,
      stripWWW: false,
      removeQueryParameters: false,
      removeTrailingSlash: true
    }));
    this.isBookmarked = !!bookmark;
    if (!noEmit) {
      this.emitUpdateState();
    }
  }

  async fetchDatInfo (noEmit = false) {
    if (!this.url.startsWith('dat://')) {
      return
    }
    var key = await beakerCore.dat.dns.resolveName(this.url);
    this.datInfo = await beakerCore.dat.library.getArchiveInfo(key);
    this.peers = this.datInfo.peers;
    this.donateLinkHref = _get(this, 'datInfo.links.payment.0.href');
    this.localPath = _get(this, 'datInfo.userSettings.localSyncPath');
    if (!noEmit) {
      this.emitUpdateState();
    }
  }

  // events
  // =

  emitUpdateState () {
    emitUpdateState(this.browserWindow, this);
  }

  onDidStartLoading (e) {
    // update state
    this.isLoading = true;
    this.loadingURL = null;
    this.favicons = null;
    this.isReceivingAssets = false;
    this.wasDatTimeout = false;

    // emit
    this.emitUpdateState();
  }

  onDidStartNavigation (e, url$$1, isInPlace, isMainFrame) {
    if (!isMainFrame) return

    // turn off live reloading if we're leaving the domain
    if (toOrigin(url$$1) !== toOrigin(this.url)) {
      this.stopLiveReloading();
    }

    // update state
    this.loadingURL = url$$1;
    this.emitUpdateState();
  }

  onDidNavigate (e, url$$1, httpResponseCode) {
    // read zoom
    setZoomFromSitedata(this);

    // update state
    this.loadError = null;
    this.loadingURL = null;
    this.isReceivingAssets = true;
    this.fetchIsBookmarked();
    this.fetchDatInfo();
    if (httpResponseCode === 504 && url$$1.startsWith('dat://')) {
      this.wasDatTimeout = true;
    }

    // emit
    this.emitUpdateState();
  }

  onDidNavigateInPage (e) {
    this.updateHistory();
  }

  onDidStopLoading (e) {
    this.updateHistory();

    // update state
    this.isLoading = false;
    this.loadingURL = null;
    this.isReceivingAssets = false;

    // check for dat alternatives
    if (this.url.startsWith('https://')) {
      this.checkForDatAlternative(this.url);
    } else {
      this.availableAlternative = '';
    }

    // run custom renderer apps
    this.injectCustomRenderers();

    // emit
    onSetCurrentLocation(this.browserWindow, this.url);
    this.emitUpdateState();
  }

  onDidFailLoad (e, errorCode, errorDescription, validatedURL, isMainFrame) {
    // ignore if this is a subresource
    if (!isMainFrame) return

    // ignore aborts. why:
    // - sometimes, aborts are caused by redirects. no biggy
    // - if the user cancels, then we dont want to give an error screen
    if (errorDescription == 'ERR_ABORTED' || errorCode == ERR_ABORTED) return

    // also ignore non-errors
    if (errorCode == 0) return

    // update state
    var isInsecureResponse = IS_CODE_INSECURE_RESPONSE(errorCode);
    this.loadError = {isInsecureResponse, errorCode, errorDescription, validatedURL};
    this.emitUpdateState();

    // render failure page
    var errorPageHTML = errorPage(this.loadError);
    this.webContents.executeJavaScript('document.documentElement.innerHTML = \'' + errorPageHTML + '\'');
  }

  onUpdateTargetUrl (e, url$$1) {
    set(this.browserWindow, url$$1);
  }

  onPageTitleUpdated (e, title) {
    this.emitUpdateState();
  }

  onPageFaviconUpdated (e, favicons) {
    this.favicons = favicons && favicons[0] ? favicons : null;
    this.emitUpdateState();
  }

  onNewWindow (e, url$$1, frameName, disposition) {
    e.preventDefault();
    if (!this.isActive) return // only open if coming from the active tab
    var setActive = (disposition === 'foreground-tab' || disposition === 'new-window');
    create$$1(this.browserWindow, url$$1, {setActive});
  }

  onMediaChange (e) {
    // our goal with this event handler is to detect that audio is playing
    // this lets us then render an "audio playing" icon on the tab
    // for whatever reason, the event consistently precedes the "is audible" being set by at most 1s
    // so, we delay for 1s, then emit a state update
    setTimeout(() => this.emitUpdateState(), 1e3);
  }

  onFoundInPage (e, res) {
    this.currentInpageFindResults = {
      activeMatchOrdinal: res.activeMatchOrdinal,
      matches: res.matches
    };
    this.emitUpdateState();
  }
}

// exported api
// =

function setup$4 () {
  // listen for webContents messages
  electron.ipcMain.on('BEAKER_MARK_NEXT_VIEW_SCRIPTCLOSEABLE', e => {
    nextViewIsScriptCloseable = true;
    e.returnValue = true;
  });
  electron.ipcMain.on('BEAKER_SCRIPTCLOSE_SELF', e => {
    var browserView = electron.BrowserView.fromWebContents(e.sender);
    if (browserView) {
      var view = findView(browserView);
      if (view && view.isScriptClosable) {
        remove(view.browserWindow, view);
        e.returnValue = true;
        return
      }
    }
    e.returnValue = false;
  });

  // track peer-counts
  beakerCore.dat.library.createEventStream().on('data', ([evt, {details}]) => {
    if (evt !== 'network-changed') return
    for (let winId in activeViews) {
      for (let view of activeViews[winId]) {
        if (view.datInfo && view.datInfo.url === details.url) {
          // update peer count
          view.peers = details.connections;
          view.emitUpdateState();
        }
        if (view.wasDatTimeout && view.url.startsWith(details.url)) {
          // refresh if this was a timed-out dat site (peers have been found)
          view.webContents.reload();
        }
      }
    }
  });
}

function getAll (win) {
  win = getTopWindow(win);
  return activeViews[win.id] || []
}

function getByIndex (win, index) {
  win = getTopWindow(win);
  if (index === 'active') return getActive(win)
  return getAll(win)[index]
}

function getIndexOfView (win, view) {
  win = getTopWindow(win);
  return getAll(win).indexOf(view)
}

function getAllPinned (win) {
  win = getTopWindow(win);
  return getAll(win).filter(p => p.isPinned)
}

function getActive (win) {
  win = getTopWindow(win);
  return getAll(win).find(view => view.isActive)
}

function findView (browserView) {
  for (let winId in activeViews) {
    for (let v of activeViews[winId]) {
      if (v.browserView === browserView) {
        return v
      }
    }
  }
}

function findContainingWindow (browserView) {
  for (let winId in activeViews) {
    for (let v of activeViews[winId]) {
      if (v.browserView === browserView) {
        return v.browserWindow
      }
    }
  }
}

function create$$1 (win, url$$1, opts = {setActive: false, isPinned: false, focusLocationBar: false}) {
  win = getTopWindow(win);
  url$$1 = url$$1 || DEFAULT_URL;
  var view = new View(win, {isPinned: opts.isPinned});

  activeViews[win.id] = activeViews[win.id] || [];
  if (opts.isPinned) {
    activeViews[win.id].splice(indexOfLastPinnedView(win), 0, view);
  } else {
    activeViews[win.id].push(view);
  }

  view.loadURL(url$$1);

  // make active if requested, or if none others are
  if (opts.setActive || !getActive(win)) {
    setActive(win, view);
  }
  emitReplaceState(win);

  if (opts.focusLocationBar) {
    win.webContents.send('command', 'focus-location');
  }

  return view
}

async function remove (win, view) {
  win = getTopWindow(win);
  // find
  var views = getAll(win);
  var i = views.indexOf(view);
  if (i == -1) {
    return console.warn('view-manager remove() called for missing view', view)
  }

  // give the 'onbeforeunload' a chance to run
  var onBeforeUnloadReturnValue = await fireBeforeUnloadEvent(view.webContents);
  if (onBeforeUnloadReturnValue) {
    var {response} = await electron.dialog.showMessageBox({
      type: 'question',
      buttons: ['Leave', 'Stay'],
      title: 'Do you want to leave this site?',
      message: 'Changes you made may not be saved.',
      defaultId: 0,
      cancelId: 1
    });
    var leave = (response === 0);
    if (!leave) return
  }

  // save, in case the user wants to restore it
  closedURLs[win.id] = closedURLs[win.id] || [];
  closedURLs[win.id].push(view.url);

  // set new active if that was
  if (view.isActive && views.length > 1) {
    setActive(win, views[i + 1] || views[i - 1]);
  }

  // remove
  view.stopLiveReloading();
  views.splice(i, 1);
  view.destroy();

  // persist pins w/o this one, if that was
  if (view.isPinned) {
    savePins(win);
  }

  // close the window if that was the last view
  if (views.length === 0) {
    return win.close()
  }

  // emit
  emitReplaceState(win);
}

function removeAllExcept (win, view) {
  win = getTopWindow(win);
  var views = getAll(win).slice(); // .slice() to duplicate the list
  for (let v of views) {
    if (v !== view) {
      remove(win, v);
    }
  }
}

function removeAllToRightOf (win, view) {
  win = getTopWindow(win);
  while (true) {
    let views = getAll(win);
    let index = views.indexOf(view) + 1;
    if (index >= views.length) break
    remove(win, getByIndex(win, index));
  }
}

function setActive (win, view) {
  win = getTopWindow(win);
  if (typeof view === 'number') {
    view = getByIndex(win, view);
  }
  if (!view) return

  // deactivate the old view
  var active = getActive(win);
  if (active) {
    active.deactivate(true);
  }

  // activate the new view
  view.activate();
  onSetCurrentLocation(win, view.url); // give the window-menu a chance to handle the change
  emitReplaceState(win);
}

function resize (win) {
  var active = getActive(win);
  if (active) active.resize();
}

function initializeFromSnapshot (win, snapshot) {
  win = getTopWindow(win);
  for (let url$$1 of snapshot) {
    create$$1(win, url$$1);
  }
}

function takeSnapshot (win) {
  win = getTopWindow(win);
  return getAll(win)
    .filter(v => !v.isPinned)
    .map(v => v.url)
    .filter(Boolean)
}

function togglePinned (win, view) {
  win = getTopWindow(win);
  // move tab to the "end" of the pinned tabs
  var views = getAll(win);
  var oldIndex = views.indexOf(view);
  var newIndex = indexOfLastPinnedView(win);
  if (oldIndex < newIndex) newIndex--;
  views.splice(oldIndex, 1);
  views.splice(newIndex, 0, view);

  // update view state
  view.isPinned = !view.isPinned;
  emitReplaceState(win);

  // persist
  savePins(win);
}

function savePins (win) {
  win = getTopWindow(win);
  return settingsDb$2.set('pinned_tabs', JSON.stringify(getAllPinned(win).map(p => p.url)))
}

async function loadPins (win) {
  win = getTopWindow(win);
  var json = await settingsDb$2.get('pinned_tabs');
  try { JSON.parse(json).forEach(url$$1 => create$$1(win, url$$1, {isPinned: true})); }
  catch (e) {}
}

function reopenLastRemoved (win) {
  win = getTopWindow(win);
  var url$$1 = (closedURLs[win.id] || []).pop();
  if (url$$1) {
    var view = create$$1(win, url$$1);
    setActive(win, view);
    return view
  }
}

function reorder (win, oldIndex, newIndex) {
  win = getTopWindow(win);
  if (oldIndex === newIndex) {
    return
  }
  var views = getAll(win);
  var view = getByIndex(win, oldIndex);
  views.splice(oldIndex, 1);
  views.splice(newIndex, 0, view);
  emitReplaceState(win);
}

function changeActiveBy (win, offset) {
  win = getTopWindow(win);
  var views = getAll(win);
  var active = getActive(win);
  if (views.length > 1) {
    var i = views.indexOf(active);
    if (i === -1) { return console.warn('Active page is not in the pages list! THIS SHOULD NOT HAPPEN!') }

    i += offset;
    if (i < 0) i = views.length - 1;
    if (i >= views.length) i = 0;

    setActive(win, views[i]);
  }
}





function openOrFocusDownloadsPage (win) {
  win = getTopWindow(win);
  var views = getAll(win);
  var downloadsView = views.find(v => v.url.startsWith('beaker://downloads'));
  if (!downloadsView) {
    downloadsView = create$$1(win, 'beaker://downloads');
  }
  setActive(win, downloadsView);
}

function emitReplaceState (win) {
  win = getTopWindow(win);
  var state = {tabs: getWindowTabState(win), isFullscreen: win.isFullScreen()};
  emit(win, 'replace-state', state);
  win.emit('custom-pages-updated', takeSnapshot(win));
}

function emitUpdateState (win, view) {
  win = getTopWindow(win);
  var index = typeof view === 'number' ? view : getAll(win).indexOf(view);
  if (index === -1) {
    console.warn('WARNING: attempted to update state of a view not on the window');
    return
  }
  var state = getByIndex(win, index).state;
  emit(win, 'update-state', {index, state});
  win.emit('custom-pages-updated', takeSnapshot(win));
}

// rpc api
// =

rpc.exportAPI('background-process-views', viewsRPCManifest, {
  createEventStream () {
    return emitStream(getEvents(getWindow(this.sender)))
  },

  async refreshState (tab) {
    var win = getWindow(this.sender);
    var view = getByIndex(win, tab);
    if (view) {
      view.refreshState();
    }
  },

  async getState () {
    var win = getWindow(this.sender);
    return getWindowTabState(win)
  },

  async getTabState (tab, opts) {
    var win = getWindow(this.sender);
    var view = getByIndex(win, tab);
    if (view) {
      var state = Object.assign({}, view.state);
      if (opts) {
        if (opts.datInfo) state.datInfo = view.datInfo;
        if (opts.networkStats) state.networkStats = view.datInfo ? view.datInfo.networkStats : {};
        if (opts.sitePerms) state.sitePerms = await beakerCore.dbs.sitedata.getPermissions(view.url);
      }
      return state
    }
  },

  async createTab (url$$1, opts = {setActive: false, addToNoRedirects: false}) {
    if (opts.addToNoRedirects) {
      addToNoRedirects(url$$1);
    }

    var win = getWindow(this.sender);
    var view = create$$1(win, url$$1, opts);
    return getAll(win).indexOf(view)
  },

  async loadURL (index, url$$1, opts = {addToNoRedirects: false}) {
    if (opts.addToNoRedirects) {
      addToNoRedirects(url$$1);
    }

    getByIndex(getWindow(this.sender), index).loadURL(url$$1);
  },

  async closeTab (index) {
    var win = getWindow(this.sender);
    remove(win, getByIndex(win, index));
  },

  async setActiveTab (index) {
    var win = getWindow(this.sender);
    setActive(win, getByIndex(win, index));
  },

  async reorderTab (oldIndex, newIndex) {
    var win = getWindow(this.sender);
    reorder(win, oldIndex, newIndex);
  },

  async showTabContextMenu (index) {
    var win = getWindow(this.sender);
    var view = getByIndex(win, index);
    var menu = electron.Menu.buildFromTemplate([
      { label: 'New Tab', click: () => create$$1(win, null, {setActive: true}) },
      { type: 'separator' },
      { label: 'Duplicate', click: () => create$$1(win, view.url) },
      { label: (view.isPinned) ? 'Unpin Tab' : 'Pin Tab', click: () => togglePinned(win, view) },
      { label: (view.isAudioMuted) ? 'Unmute Tab' : 'Mute Tab', click: () => view.toggleMuted() },
      { type: 'separator' },
      { label: 'Close Tab', click: () => remove(win, view) },
      { label: 'Close Other Tabs', click: () => removeAllExcept(win, view) },
      { label: 'Close Tabs to the Right', click: () => removeAllToRightOf(win, view) },
      { type: 'separator' },
      { label: 'Reopen Closed Tab', click: () => reopenLastRemoved(win) }
    ]);
    menu.popup({window: win});
  },

  async showLocationBarContextMenu (index) {
    var win = getWindow(this.sender);
    var view = getByIndex(win, index);
    var clipboardContent = electron.clipboard.readText();
    var clipInfo = examineLocationInput(clipboardContent);
    var menu = electron.Menu.buildFromTemplate([
      { label: 'Cut', role: 'cut' },
      { label: 'Copy', role: 'copy' },
      { label: 'Paste', role: 'paste' },
      { label: `Paste and ${clipInfo.isProbablyUrl ? 'Go' : 'Search'}`, click: onPasteAndGo }
    ]);
    menu.popup({window: win});

    function onPasteAndGo () {
      // close the menu
      hide(win);
      win.webContents.send('command', 'unfocus-location');

      // load the URL
      var url$$1 = clipInfo.isProbablyUrl ? clipInfo.vWithProtocol : clipInfo.vSearch;
      view.loadURL(url$$1);
    }
  },

  async goBack (index) {
    getByIndex(getWindow(this.sender), index).browserView.webContents.goBack();
  },

  async goForward (index) {
    getByIndex(getWindow(this.sender), index).browserView.webContents.goForward();
  },

  async stop (index) {
    getByIndex(getWindow(this.sender), index).browserView.webContents.stop();
  },

  async reload (index) {
    getByIndex(getWindow(this.sender), index).browserView.webContents.reload();
  },

  async resetZoom (index) {
    zoomReset(getByIndex(getWindow(this.sender), index));
  },

  async toggleLiveReloading (index) {
    getByIndex(getWindow(this.sender), index).toggleLiveReloading();
  },

  async showInpageFind (index) {
    getByIndex(getWindow(this.sender), index).showInpageFind();
  },

  async hideInpageFind (index) {
    getByIndex(getWindow(this.sender), index).hideInpageFind();
  },

  async setInpageFindString (index, str, dir) {
    getByIndex(getWindow(this.sender), index).setInpageFindString(str, dir);
  },

  async moveInpageFind (index, dir) {
    getByIndex(getWindow(this.sender), index).moveInpageFind(dir);
  },

  async showLocationBar (opts) {
    await show$1(getWindow(this.sender), opts);
  },

  async hideLocationBar () {
    await hide$1(getWindow(this.sender));
  },

  async runLocationBarCmd (cmd, opts) {
    return runCmd(getWindow(this.sender), cmd, opts)
  },

  async showMenu (id, opts) {
    await show(getWindow(this.sender), id, opts);
  },

  async toggleMenu (id, opts) {
    await toggle(getWindow(this.sender), id, opts);
  },

  async focusShellWindow () {
    getWindow(this.sender).webContents.focus();
  },

  async onFaviconLoadSuccess (index, dataUrl) {
    var view = getByIndex(getWindow(this.sender), index);
    if (view) {
      beakerCore.dbs.sitedata.set(view.url, 'favicon', dataUrl);
    }
  },

  async onFaviconLoadError (index) {
    var view = getByIndex(getWindow(this.sender), index);
    if (view) {
      view.favicons = null;
      view.emitUpdateState();
    }
  }
});

// internal methods
// =

function getWindow (sender) {
  return getTopWindow(electron.BrowserWindow.fromWebContents(sender))
}

// helper ensures that if a subwindow is called, we use the parent
function getTopWindow (win) {
  while (win.getParentWindow()) {
    win = win.getParentWindow();
  }
  return win
}

function getEvents (win) {
  if (!(win.id in windowEvents)) {
    windowEvents[win.id] = new Events();
  }
  return windowEvents[win.id]
}

function emit (win, ...args) {
  getEvents(win).emit(...args);
}

function getWindowTabState (win) {
  return getAll(win).map(view => view.state)
}

function indexOfLastPinnedView (win) {
  var views = getAll(win);
  var index = 0;
  for (index; index < views.length; index++) {
    if (!views[index].isPinned) break
  }
  return index
}

function toOrigin (str) {
  try {
    var u = new URL(str);
    return u.protocol + '//' + u.hostname
  } catch (e) { return '' }
}

function addToNoRedirects (url$$1) {
  try {
    var u = new URL(url$$1);
    noRedirectHostnames.add(u.hostname);
  } catch (e) {
    console.log('Failed to add URL to noRedirectHostnames', url$$1, e);
  }
}

async function fireBeforeUnloadEvent (wc) {
  try {
    if (wc.isLoading() || wc.isWaitingForResponse()) {
      return // dont bother
    }
    return await wc.executeJavaScript(`
      (function () {
        let unloadEvent = new Event('beforeunload', {bubbles: false, cancelable: true})
        unloadEvent.returnValue = false
        return window.dispatchEvent(unloadEvent)
      })()
    `)
    } catch (e) {
      // ignore
    }
}

// `nextViewIsScriptCloseable` is set by a message received prior to window.open() being called
// we capture the state of the flag on the next created view, then reset it
function takeIsScriptClosable () {
  var b = nextViewIsScriptCloseable;
  nextViewIsScriptCloseable = false;
  return b
}

/*
The webviews that run untrusted content, by default, will handle all key press events.
The webview handlers take precedence over the browser keybindings (which are done in the window-menu).
To avoid that, we listen to the window webContents' 'before-input-event' and handle the commands manually.
*/

const IS_DARWIN$1 = process.platform === 'darwin';
const KEYBINDINGS = extractKeybindings(buildWindowMenu());
const registeredKBs = {}; // map of [window.id] => keybindings

// exported api
// =

function registerGlobalKeybinding (win, accelerator, callback) {
  // sanity checks
  checkAccelerator(accelerator);

  // add the keybinding
  registeredKBs[win.id] = registeredKBs[win.id] || [];
  registeredKBs[win.id].push({
    eventStamp: keyboardeventFromElectronAccelerator.toKeyEvent(accelerator),
    callback,
    enabled: true
  });
}



// event handler for global shortcuts
function createGlobalKeybindingsHandler (win) {
  return (e, input) => {
    if (input.type === 'keyUp') return
    var event = normalizeEvent(input);
    for (let {eventStamp, callback} of (registeredKBs[win.id] || [])) {
      if (equals(eventStamp, event)) {
        callback();
        return
      }
    }
  }
}

// event handler, manually run any events that match the window-menu's shortcuts and which are marked as 'reserved'
// this is used, for instance, to reserve "Cmd/Ctrl + T" so that an app cannot pre-empt it
// (the window-menu's accelerators are typically handled *after* the active view's input handlers)
function createKeybindingProtectionsHandler (win) {
  return (e, input) => {
    if (input.type !== 'keyDown') return
    var key = input.key;
    if (key === 'Dead') key = 'i'; // not... really sure what 'Dead' is about -prf
    if (key === '=') key = '+'; // let's not differentiate the shift (see #1155) -prf
    var match;
    for (var kb of KEYBINDINGS) {
      if (key === kb.binding.key) {
        if (kb.binding.control && !input.control) continue
        if (kb.binding.cmd && !input.meta) continue
        if (kb.binding.shift && !input.shift) continue
        if (kb.binding.alt && !input.alt) continue
        match = kb;
      }
    }
    if (match) {
      e.preventDefault();
      match.cmd(null, win);
    }
  }
}

// internal
// =

// recurse the window menu and extract all 'accelerator' values with reserved=true
function extractKeybindings (menuNode) {
  if (menuNode.accelerator && menuNode.click && menuNode.reserved) {
    return {
      binding: convertAcceleratorToBinding(menuNode.accelerator),
      cmd: menuNode.click
    }
  } else if (menuNode.submenu) {
    return menuNode.submenu.map(extractKeybindings).filter(Boolean)
  } else if (Array.isArray(menuNode)) {
    return _flattenDeep(menuNode.map(extractKeybindings).filter(Boolean))
  }
  return null
}

// convert accelerator values into objects that are easy to match against input events
// eg 'CmdOrCtrl+Shift+T' -> {cmdOrCtrl: true, shift: true, key: 't'}
function convertAcceleratorToBinding (accel) {
  var binding = {};
  accel.split('+').forEach(part => {
    switch (part.toLowerCase()) {
      case 'command':
      case 'cmd':
        binding.cmd = true;
        break
      case 'ctrl':
        binding.control = true;
        break
      case 'cmdorctrl':
        if (IS_DARWIN$1) binding.cmd = true;
        else binding.control = true;
        break
      case 'alt':
        binding.alt = true;
        break
      case 'shift':
        binding.shift = true;
        break
      case 'plus':
        binding.key = '+';
        break
      default:
        binding.key = part.toLowerCase();
    }
  });
  return binding
}

function checkAccelerator (accelerator) {
  if (!isAccelerator(accelerator)) {
    throw new Error(`${accelerator} is not a valid accelerator`)
  }
}

function normalizeEvent (input) {
  var normalizedEvent = {
    code: input.code,
    key: input.key
  };

  for (let prop of ['alt', 'shift', 'meta']) {
    if (typeof input[prop] !== 'undefined') {
      normalizedEvent[`${prop}Key`] = input[prop];
    }
  }

  if (typeof input.control !== 'undefined') {
    normalizedEvent.ctrlKey = input.control;
  }

  return normalizedEvent
}

const settingsDb$1 = beakerCore.dbs.settings;

const IS_WIN = process.platform === 'win32';
const subwindows = {
  statusBar: statusBarSubwindow,
  locationBar: locationBarSubwindow,
  menu: shellMenusSubwindow,
  permPrompt: permPromptSubwindow,
  modals: modalsSubwindow
};

// globals
// =
let userDataDir;
let numActiveWindows = 0;
let firstWindow = null;
let sessionWatcher = null;
let focusedDevtoolsHost;
let hasFirstWindowLoaded = false;
const BROWSING_SESSION_PATH = './shell-window-state.json';
const ICON_PATH = path__default.join(__dirname, (process.platform === 'win32') ? './assets/img/logo.ico' : './assets/img/logo.png');
const PRELOAD_PATH = path__default.join(__dirname, 'shell-window.build.js');

// exported methods
// =

async function setup$3 () {
  // config
  userDataDir = jetpack.cwd(electron.app.getPath('userData'));

  // set up app events
  electron.app.on('activate', () => {
    // wait for ready (not waiting can trigger errors)
    if (electron.app.isReady()) ensureOneWindowExists();
    else electron.app.on('ready', ensureOneWindowExists);
  });
  electron.ipcMain.on('new-window', () => createShellWindow());
  electron.app.on('custom-window-all-closed', () => {
    if (process.platform !== 'darwin') electron.app.quit();
  });

  setup$2();
  setup$4();

  electron.app.on('before-quit', async e => {
    sessionWatcher.exit();
    sessionWatcher.stopRecording();
  });

  electron.app.on('web-contents-created', async (e, wc) => {
    // await setup
    await new Promise(resolve => wc.once('dom-ready', resolve));

    // handle shell-window webcontents
    const window = electron.BrowserWindow.fromWebContents(wc);
    if (window) {
      // attach global keybindings
      wc.on('before-input-event', createGlobalKeybindingsHandler(window));
      return
    }

    // handle tab webcontents
    const parentView = electron.BrowserView.fromWebContents(wc);
    if (!parentView) return
    const parentWindow = findContainingWindow(parentView);
    if (!parentWindow) return

    // attach global keybindings
    wc.on('before-input-event', createGlobalKeybindingsHandler(parentWindow));
    // attach keybinding protections
    wc.on('before-input-event', createKeybindingProtectionsHandler(parentWindow));

    // HACK
    // add link-click handling to page devtools
    // (it would be much better to update Electron to support this, rather than overriding like this)
    // -prf
    wc.on('devtools-opened', () => {
      if (wc.devToolsWebContents) {
        wc.devToolsWebContents.executeJavaScript('InspectorFrontendHost.openInNewTab = (url) => window.open(url)');
        wc.devToolsWebContents.on('new-window', (e, url$$1) => {
          if (url$$1.startsWith('chrome-devtools://')) return // ignore
          create$$1(parentWindow, url$$1, {setActive: true});
        });
      }
    });

    // track focused devtools host
    wc.on('devtools-focused', () => { focusedDevtoolsHost = wc; });
    wc.on('devtools-closed', unfocusDevtoolsHost);
    wc.on('destroyed', unfocusDevtoolsHost);
    function unfocusDevtoolsHost () {
      if (focusedDevtoolsHost === wc) {
        focusedDevtoolsHost = undefined;
      }
    }
  });

  let previousSessionState = getPreviousBrowsingSession();
  sessionWatcher = new SessionWatcher(userDataDir);
  let customStartPage = await settingsDb$1.get('custom_start_page');
  let isTestDriverActive = !!beakerCore.getEnvVar('BEAKER_TEST_DRIVER');
  let isOpenUrlEnvVar = !!beakerCore.getEnvVar('BEAKER_OPEN_URL');

  if (!isTestDriverActive && !isOpenUrlEnvVar && (customStartPage === 'previous' || (!previousSessionState.cleanExit && userWantsToRestoreSession()))) {
    // restore old window
    restoreBrowsingSession(previousSessionState);
  } else {
    let opts = {};
    if (previousSessionState.windows[0]) {
      // use the last session's window position
      let {x, y, width, height} = previousSessionState.windows[0];
      opts.x = x;
      opts.y = y;
      opts.width = width;
      opts.height = height;
    }
    if (isOpenUrlEnvVar) {
      // use the env var if specified
      opts.pages = [beakerCore.getEnvVar('BEAKER_OPEN_URL')];
    }
    // create new window
    createShellWindow(opts);
  }
}

function createShellWindow (windowState) {
  // create window
  let state = ensureVisibleOnSomeDisplay(Object.assign({}, defaultWindowState(), windowState));
  var { x, y, width, height, minWidth, minHeight } = state;
  var win = new electron.BrowserWindow({
    titleBarStyle: 'hiddenInset',
    autoHideMenuBar: true,
    fullscreenable: true,
    fullscreenWindowTitle: true,
    frame: !IS_WIN,
    x,
    y,
    width,
    height,
    minWidth,
    minHeight,
    backgroundColor: '#ddd',
    webPreferences: {
      preload: PRELOAD_PATH,
      defaultEncoding: 'utf-8',
      nodeIntegration: false,
      contextIsolation: false,
      webviewTag: false,
      sandbox: true,
      webSecurity: false, // disable same-origin-policy in the shell window, webviews have it restored
      // enableRemoteModule: false, TODO would prefer this were true, but shell window needs this to get the webviews' webContents IDs -prf
      allowRunningInsecureContent: false,
      nativeWindowOpen: true
    },
    icon: ICON_PATH,
    show: false // will show when ready
  });
  win.once('ready-to-show', () => {
    win.show();
    if (!hasFirstWindowLoaded) {
      hasFirstWindowLoaded = true;
      electron.app.emit('custom-ready-to-show');
    }
  });
  for (let k in subwindows) {
    subwindows[k].setup(win);
  }
  registerListener(win);
  win.loadURL('beaker://shell-window');
  sessionWatcher.watchWindow(win, state);

  function handlePagesReady ({ sender }) {
    if (!win || win.isDestroyed()) return

    if (sender === win.webContents) {
      if (win.webContents.id === firstWindow) {
        // if this is the first window opened (since app start or since all windows closing)
        loadPins(win);
      }
      initializeFromSnapshot(win, state.pages);
    }
  }

  numActiveWindows++;

  if (numActiveWindows === 1) {
    firstWindow = win.webContents.id;
  }

  electron.ipcMain.on('shell-window:ready', handlePagesReady);
  win.on('closed', () => {
    electron.ipcMain.removeListener('shell-window:ready', handlePagesReady);
    for (let k in subwindows) {
      subwindows[k].destroy(win);
    }
  });

  // register shortcuts
  for (var i = 1; i <= 8; i++) { registerGlobalKeybinding(win, 'CmdOrCtrl+' + i, onTabSelect(win, i - 1)); }
  registerGlobalKeybinding(win, 'CmdOrCtrl+9', onLastTab(win));
  registerGlobalKeybinding(win, 'Ctrl+Tab', onNextTab(win));
  registerGlobalKeybinding(win, 'Ctrl+Shift+Tab', onPrevTab(win));
  registerGlobalKeybinding(win, 'Ctrl+PageUp', onPrevTab(win));
  registerGlobalKeybinding(win, 'Ctrl+PageDown', onNextTab(win));
  registerGlobalKeybinding(win, 'CmdOrCtrl+[', onGoBack(win));
  registerGlobalKeybinding(win, 'CmdOrCtrl+]', onGoForward(win));
  registerGlobalKeybinding(win, 'Alt+D', onFocusLocation(win));
  registerGlobalKeybinding(win, 'F5', onReload(win));
  registerGlobalKeybinding(win, 'F6', onFocusLocation(win));

  // register event handlers
  win.on('browser-backward', onGoBack(win));
  win.on('browser-forward', onGoForward(win));
  win.on('scroll-touch-begin', sendScrollTouchBegin);
  win.on('scroll-touch-end', sendToWebContents('scroll-touch-end'));
  win.on('focus', sendToWebContents('focus'));
  win.on('blur', e => {
    set(win, false); // hide the statusbar on blur
    sendToWebContents('blur')(e);
  });
  win.on('app-command', (e, cmd) => { onAppCommand(win, e, cmd); });
  win.on('enter-full-screen', e => {
    // update UI
    emitReplaceState(win);

    // TODO
    // registerGlobalKeybinding(win, 'Esc', onEscape(win))
    // sendToWebContents('enter-full-screen')(e)
  });
  win.on('leave-full-screen', e => {
    // update UI
    emitReplaceState(win);

    // TODO
    // unregisterGlobalKeybinding(win, 'Esc')
    // sendToWebContents('leave-full-screen')(e)
  });
  win.on('resize', () => {
    resize(win);
    for (let k in subwindows) {
      subwindows[k].reposition(win);
    }
  });
  win.on('move', () => {
    for (let k in subwindows) {
      subwindows[k].reposition(win);
    }
  });
  win.on('close', onClose(win));

  return win
}

function getActiveWindow () {
  // try to pull the focused window; if there isnt one, fallback to the last created
  var win = electron.BrowserWindow.getFocusedWindow();
  if (!win || win.webContents.getURL() !== 'beaker://shell-window/') {
    win = electron.BrowserWindow.getAllWindows().filter(win => win.webContents.getURL() === 'beaker://shell-window/').pop();
  }
  return win
}

function getFocusedDevToolsHost () {
  // check first if it's the shell window's devtools
  let win = electron.BrowserWindow.getAllWindows().find(w => w.webContents.isDevToolsFocused());
  if (win) return win.webContents
  // fallback to our manually tracked devtools host
  return focusedDevtoolsHost
}



function ensureOneWindowExists () {
  if (numActiveWindows === 0) {
    createShellWindow();
  }
}

// internal methods
// =

function windowWithinBounds (windowState, bounds) {
  return windowState.x >= bounds.x &&
    windowState.y >= bounds.y &&
    windowState.x + windowState.width <= bounds.x + bounds.width &&
    windowState.y + windowState.height <= bounds.y + bounds.height
}

function userWantsToRestoreSession () {
  let answer = electron.dialog.showMessageBoxSync({
    type: 'question',
    message: 'Sorry! It looks like Beaker crashed',
    detail: 'Would you like to restore your previous browsing session?',
    buttons: [ 'Restore Session', 'Start New Session' ],
    defaultId: 0,
    icon: ICON_PATH
  });
  return answer === 0
}

function restoreBrowsingSession (previousSessionState) {
  let { windows } = previousSessionState;
  if (windows.length) {
    for (let windowState of windows) {
      if (windowState) createShellWindow(windowState);
    }
  } else {
    createShellWindow();
  }
}

function getPreviousBrowsingSession () {
  var restoredState = {};
  try {
    restoredState = userDataDir.read(BROWSING_SESSION_PATH, 'json');
  } catch (err) {
    // For some reason json can't be read (might be corrupted).
    // No worries, we have defaults.
  }
  return Object.assign({}, defaultBrowsingSessionState(), restoredState)
}

function ensureVisibleOnSomeDisplay (windowState) {
  // HACK
  // for some reason, electron.screen comes back null sometimes
  // not sure why, shouldn't be happening
  // check for existence for now, see #690
  // -prf
  const screen$$1 = getScreenAPI();
  var visible = screen$$1 && screen$$1.getAllDisplays().some(display => windowWithinBounds(windowState, display.bounds));
  if (!visible) {
    // Window is partially or fully not visible now.
    // Reset it to safe defaults.
    return defaultWindowState(windowState)
  }
  return windowState
}

// shortcut event handlers
// =

function onClose (win) {
  return e => {
    numActiveWindows--;
    if (numActiveWindows === 0) {
      // emit a custom 'window-all-closed'
      // we need to do this because we have hidden windows running additional behaviors
      electron.app.emit('custom-window-all-closed');
    }

    // deny any outstanding permission requests
    denyAllRequests(win);
  }
}

function onTabSelect (win, tabIndex) {
  return () => setActive(win, tabIndex)
}

function onLastTab (win) {
  return () => setActive(win, getAll(win).slice(-1)[0])
}

function onNextTab (win) {
  return () => changeActiveBy(win, 1)
}

function onPrevTab (win) {
  return () => changeActiveBy(win, -1)
}

function onGoBack (win) {
  return () => getActive(win).webContents.goBack()
}

function onGoForward (win) {
  return () => getActive(win).webContents.goForward()
}

function onReload (win) {
  return () => getActive(win).webContents.reload()
}

function onFocusLocation (win) {
  return () => win.webContents.send('command', 'focus-location')
}

function onAppCommand (win, e, cmd) {
  // handles App Command events (Windows)
  // see https://electronjs.org/docs/all#event-app-command-windows
  switch (cmd) {
    case 'browser-backward':
      getActive(win).webContents.goBack();
      break
    case 'browser-forward':
      getActive(win).webContents.goForward();
      break
    default:
      break
  }
}

// window event handlers
// =

function sendToWebContents (event) {
  return e => e.sender.webContents.send('window-event', event)
}

function sendScrollTouchBegin (e) {
  // get the cursor x/y within the window
  const screen$$1 = getScreenAPI();
  if (!screen$$1) return
  var cursorPos = screen$$1.getCursorScreenPoint();
  var winPos = e.sender.getBounds();
  cursorPos.x -= winPos.x; cursorPos.y -= winPos.y;
  e.sender.webContents.send('window-event', 'scroll-touch-begin', {
    cursorX: cursorPos.x,
    cursorY: cursorPos.y
  });
}

// helpers
// =

function getScreenAPI () {
  return require('electron').screen
}

// handle OSX open-url event
var queue = [];
var isLoaded = false;
var isSetup = false;

function setup$2 () {
  if (isSetup) return
  isSetup = true;
  electron.ipcMain.on('shell-window:ready', function (e) {
    var win = electron.BrowserWindow.fromWebContents(e.sender);
    queue.forEach(url$$1 => create$$1(win, url$$1));
    queue.length = 0;
    isLoaded = true;
  });
}

function open (url$$1) {
  setup$2();
  var win = getActiveWindow();
  if (isLoaded && win) {
    // send command now
    create$$1(win, url$$1);
    win.show();
  } else {
    // queue for later
    queue.push(url$$1);

    // no longer loaded?
    if (isLoaded) {
      isLoaded = false;
      // spawn a new window
      createShellWindow();
    }
  }
  return win && win.webContents
}

const exec = require('util').promisify(require('child_process').exec);
const debug = beakerCore.debugLogger('beaker');
const settingsDb = beakerCore.dbs.settings;
// constants
// =

const IS_FROM_SOURCE = (process.defaultApp || /node_modules[\\/]electron[\\/]/.test(process.execPath));
const IS_LINUX = !(/^win/.test(process.platform)) && process.platform !== 'darwin';
const DOT_DESKTOP_FILENAME = 'appimagekit-beaker-browser.desktop';
const isBrowserUpdatesSupported = !(IS_LINUX || IS_FROM_SOURCE); // linux is temporarily not supported

// how long between scheduled auto updates?
const SCHEDULED_AUTO_UPDATE_DELAY = 24 * 60 * 60 * 1e3; // once a day

// possible updater states
const UPDATER_STATUS_IDLE = 'idle';
const UPDATER_STATUS_CHECKING = 'checking';
const UPDATER_STATUS_DOWNLOADING = 'downloading';
const UPDATER_STATUS_DOWNLOADED = 'downloaded';

// globals
// =

// dont automatically check for updates (need to respect user preference)
electronUpdater.autoUpdater.autoDownload = false;

// what's the updater doing?
var updaterState = UPDATER_STATUS_IDLE;
var updaterError = false; // has there been an error?

// where is the user in the setup flow?
var userSetupStatus = false;
var userSetupStatusLookupPromise;

// content-type tracker
var resourceContentTypes = new LRU(100); // URL -> Content-Type

// events emitted to rpc clients
var browserEvents = new Events();

process.on('unhandledRejection', (reason, p) => {
  console.error('Unhandled Rejection at: Promise', p, 'reason:', reason);
  debug('Unhandled Rejection at: Promise', p, 'reason:', reason);
});
process.on('uncaughtException', (err) => {
  console.error('Uncaught exception:', err);
  debug('Uncaught exception:', err);
});

// exported methods
// =

function setup$1 () {
  // setup auto-updater
  if (isBrowserUpdatesSupported) {
    try {
      electronUpdater.autoUpdater.setFeedURL(getAutoUpdaterFeedSettings());
      electronUpdater.autoUpdater.on('update-available', onUpdateAvailable);
      electronUpdater.autoUpdater.on('update-not-available', onUpdateNotAvailable);
      electronUpdater.autoUpdater.on('update-downloaded', onUpdateDownloaded);
      electronUpdater.autoUpdater.on('error', onUpdateError);
    } catch (e) {
      debug('[AUTO-UPDATE] error', e.toString());
    }
    setTimeout(scheduledAutoUpdate, 15e3); // wait 15s for first run
  }

  // fetch user setup status
  userSetupStatusLookupPromise = settingsDb.get('user-setup-status');

  // wire up events
  electron.app.on('web-contents-created', onWebContentsCreated);

  // window.prompt handling
  //  - we have use ipc directly instead of using rpc, because we need custom
  //    response-lifecycle management in the main thread
  electron.ipcMain.on('page-prompt-dialog', async (e, message, def) => {
    try {
      var res = await create$2(e.sender, 'prompt', {message, default: def});
      e.returnValue = res && res.value ? res.value : false;
    } catch (e) {
      e.returnValue = false;
    }
  });

  // HACK
  // Electron doesn't give us a convenient way to check the content-types of responses
  // so we track the last 100 responses' headers to accomplish this
  // -prf
  electron.session.defaultSession.webRequest.onCompleted(onCompleted);
}

const WEBAPI = {
  createEventsStream,
  getInfo,
  checkForUpdates,
  restartBrowser,

  getSetting,
  getSettings,
  setSetting,
  getUserSetupStatus,
  setUserSetupStatus,
  getDefaultLocalPath,
  setStartPageBackgroundImage,
  getDefaultProtocolSettings,
  setAsDefaultProtocolClient,
  removeAsDefaultProtocolClient,

  fetchBody,
  downloadURL,

  getResourceContentType,

  listBuiltinFavicons,
  getBuiltinFavicon,
  uploadFavicon,
  imageToIco,

  setWindowDimensions,
  showOpenDialog,
  showContextMenu,
  openUrl: url$$1 => { open(url$$1); }, // dont return anything
  openFolder,
  doWebcontentsCmd,
  doTest,
  closeModal: () => {} // DEPRECATED, probably safe to remove soon
};

function fetchBody (url$$1) {
  return new Promise((resolve) => {
    var http = url$$1.startsWith('https') ? require('https') : require('http');

    http.get(url$$1, (res) => {
      var body = '';
      res.setEncoding('utf8');
      res.on('data', (data) => { body += data; });
      res.on('end', () => resolve(body));
    });
  })
}

async function downloadURL (url$$1) {
  this.sender.downloadURL(url$$1);
}

function getResourceContentType (url$$1) {
  let i = url$$1.indexOf('#');
  if (i !== -1) url$$1 = url$$1.slice(0, i); // strip the fragment
  return resourceContentTypes.get(url$$1)
}

async function listBuiltinFavicons ({filter, offset, limit} = {}) {
  if (filter) {
    filter = new RegExp(filter, 'i');
  }

  // list files in assets/favicons and filter on the name
  var dir = jetpack.cwd(__dirname).cwd('assets/favicons');
  var items = (await dir.listAsync())
    .filter(filename => {
      if (filter && !filter.test(filename)) {
        return false
      }
      return filename.endsWith('.ico')
    });
  return items.slice(offset || 0, limit || Number.POSITIVE_INFINITY)
}

async function getBuiltinFavicon (name) {
  var dir = jetpack.cwd(__dirname).cwd('assets/favicons');
  return dir.readAsync(name, 'buffer')
}

async function uploadFavicon () {
  let favicon = electron.dialog.showOpenDialogSync({
    title: 'Upload Favicon...',
    defaultPath: electron.app.getPath('home'),
    buttonLabel: 'Upload Favicon',
    filters: [
      { name: 'Images', extensions: ['png', 'ico', 'jpg'] }
    ],
    properties: ['openFile']
  });

  if (!favicon) return

  let faviconBuffer = await jetpack.readAsync(favicon[0], 'buffer');
  let extension = path__default.extname(favicon[0]);

  if (extension === '.png') {
    return toIco(faviconBuffer, {resize: true})
  }
  if (extension === '.jpg') {
    let imageToPng = electron.nativeImage.createFromBuffer(faviconBuffer).toPNG();
    return toIco(imageToPng, {resize: true})
  }
  if (extension === '.ico' && ICO.isICO(faviconBuffer)) {
    return faviconBuffer
  }
}

async function imageToIco (image) {
  // TODO expand on this function to be png/jpg to ico
  let imageToPng = electron.nativeImage.createFromDataURL(image).toPNG();
  return toIco(imageToPng, {resize: true})
}

async function setWindowDimensions ({width, height} = {}) {
  var wc = this.sender;
  while (wc.hostWebContents) wc = wc.hostWebContents;
  var win = electron.BrowserWindow.fromWebContents(wc);
  var [currentWidth, currentHeight] = win.getSize();
  width = width || currentWidth;
  height = height || currentHeight;
  win.setSize(width, height);
}

function setStartPageBackgroundImage (srcPath, appendCurrentDir) {
  if (appendCurrentDir) {
    srcPath = path__default.join(__dirname, `/${srcPath}`);
  }

  var destPath = path__default.join(electron.app.getPath('userData'), 'start-background-image');

  return new Promise((resolve) => {
    if (srcPath) {
      fs__default.readFile(srcPath, (_, data) => {
        fs__default.writeFile(destPath, data, () => resolve());
      });
    } else {
      fs__default.unlink(destPath, () => resolve());
    }
  })
}

async function getDefaultProtocolSettings () {
  if (IS_LINUX) {
    // HACK
    // xdb-settings doesnt currently handle apps that you can't `which`
    // we can just use xdg-mime directly instead
    // see https://github.com/beakerbrowser/beaker/issues/915
    // -prf
    let [httpHandler, datHandler] = await Promise.all([
      // If there is no default specified, be sure to catch any error
      // from exec and return '' otherwise Promise.all errors out.
      exec('xdg-mime query default "x-scheme-handler/http"').catch(err => ''),
      exec('xdg-mime query default "x-scheme-handler/dat"').catch(err => '')
    ]);
    if (httpHandler && httpHandler.stdout) httpHandler = httpHandler.stdout;
    if (datHandler && datHandler.stdout) datHandler = datHandler.stdout;
    return {
      http: (httpHandler || '').toString().trim() === DOT_DESKTOP_FILENAME,
      dat: (datHandler || '').toString().trim() === DOT_DESKTOP_FILENAME
    }
  }

  return Promise.resolve(['http', 'dat'].reduce((res, x) => {
    res[x] = electron.app.isDefaultProtocolClient(x);
    return res
  }, {}))
}

async function setAsDefaultProtocolClient (protocol$$1) {
  if (IS_LINUX) {
    // HACK
    // xdb-settings doesnt currently handle apps that you can't `which`
    // we can just use xdg-mime directly instead
    // see https://github.com/beakerbrowser/beaker/issues/915
    // -prf
    await exec(`xdg-mime default ${DOT_DESKTOP_FILENAME} "x-scheme-handler/${protocol$$1}"`);
    return true
  }
  return Promise.resolve(electron.app.setAsDefaultProtocolClient(protocol$$1))
}

function removeAsDefaultProtocolClient (protocol$$1) {
  return Promise.resolve(electron.app.removeAsDefaultProtocolClient(protocol$$1))
}

function getInfo () {
  return {
    version: electron.app.getVersion(),
    electronVersion: process.versions.electron,
    chromiumVersion: process.versions.chrome,
    nodeVersion: process.versions.node,
    platform: os.platform(),
    updater: {
      isBrowserUpdatesSupported,
      error: updaterError,
      state: updaterState
    },
    paths: {
      userData: electron.app.getPath('userData')
    }
  }
}

function checkForUpdates (opts = {}) {
  // dont overlap
  if (updaterState != UPDATER_STATUS_IDLE) { return }

  // update global state
  debug('[AUTO-UPDATE] Checking for a new version.');
  updaterError = false;
  setUpdaterState(UPDATER_STATUS_CHECKING);
  if (opts.prerelease) {
    debug('[AUTO-UPDATE] Jumping to pre-releases.');
    electronUpdater.autoUpdater.allowPrerelease = true;
  }
  electronUpdater.autoUpdater.checkForUpdates();

  // just return a resolve; results will be emitted
  return Promise.resolve()
}

function restartBrowser () {
  if (updaterState == UPDATER_STATUS_DOWNLOADED) {
    // run the update installer
    electronUpdater.autoUpdater.quitAndInstall();
    debug('[AUTO-UPDATE] Quitting and installing.');
  } else {
    debug('Restarting Beaker by restartBrowser()');
    // do a simple restart
    electron.app.relaunch();
    setTimeout(() => electron.app.exit(0), 1e3);
  }
}

function getSetting (key) {
  return settingsDb.get(key)
}

function getSettings () {
  return settingsDb.getAll()
}

function setSetting (key, value) {
  return settingsDb.set(key, value)
}

async function getUserSetupStatus () {
  // if not cached, defer to the lookup promise
  return (userSetupStatus) || userSetupStatusLookupPromise
}

function setUserSetupStatus (status) {
  userSetupStatus = status; // cache
  return settingsDb.set('user-setup-status', status)
}

const SCROLLBAR_WIDTH = 16;
async function capturePage (url$$1, opts) {
  var width = opts.width || 1024;
  var height = opts.height || 768;

  var win = new electron.BrowserWindow({
    width: width + SCROLLBAR_WIDTH,
    height,
    show: false,
    defaultEncoding: 'utf-8',
    partition: 'session-' + Date.now() + Math.random(),
    preload: 'file://' + path__default.join(electron.app.getAppPath(), 'webview-preload.build.js'),
    webPreferences: {
      webSecurity: true,
      allowRunningInsecureContent: false,
      nativeWindowOpen: true
    }
  });
  win.loadURL(url$$1);

  // wait for load
  await new Promise((resolve, reject) => {
    win.webContents.on('did-finish-load', resolve);
  });
  await new Promise(r => setTimeout(r, 200)); // give an extra 200ms for rendering

  // capture the page
  var image = await new Promise((resolve, reject) => {
    win.webContents.capturePage({x: 0, y: 0, width, height}, resolve);
  });

  // resize if asked
  if (opts.resizeTo) {
    image = image.resize(opts.resizeTo);
  }

  return image.toPNG()
}

// rpc methods
// =

function createEventsStream () {
  return emitStream(browserEvents)
}

function showOpenDialog (opts = {}) {
  var wc = this.sender.webContents;
  if (wc.hostWebContents) {
    wc = wc.hostWebContents;
  }
  return new Promise((resolve) => {
    electron.dialog.showOpenDialog({
      title: opts.title,
      buttonLabel: opts.buttonLabel,
      filters: opts.filters,
      properties: opts.properties,
      defaultPath: opts.defaultPath
    }).then(({filePaths}) => {
      // return focus back to the the webview
      wc.executeJavaScript(`
        var wv = document.querySelector('webview:not(.hidden)')
        if (wv) wv.focus()
      `);
      resolve(filePaths);
    }, err => {
      resolve([]);
    });
  })
}

function showContextMenu (menuDefinition) {
  return new Promise(resolve => {
    var cursorPos = electron.screen.getCursorScreenPoint();

    // add a click item to all menu items
    addClickHandler(menuDefinition);
    function addClickHandler (items) {
      items.forEach(item => {
        if (item.type === 'submenu' && Array.isArray(item.submenu)) {
          addClickHandler(item.submenu);
        } else if (item.type !== 'separator' && item.id) {
          item.click = clickHandler;
        }
      });
    }

    // add 'inspect element' in development
    if (beakerCore.getEnvVar('NODE_ENV') === 'develop' || beakerCore.getEnvVar('NODE_ENV') === 'test') {
      menuDefinition.push({type: 'separator'});
      menuDefinition.push({
        label: 'Inspect Element',
        click: () => {
          this.sender.inspectElement(cursorPos.x, cursorPos.y);
          if (this.sender.isDevToolsOpened()) { this.sender.devToolsWebContents.focus(); }
        }
      });
    }

    // track the selection
    var selection;
    function clickHandler (item) {
      selection = item.id;
    }

    // show the menu
    var win = electron.BrowserWindow.fromWebContents(this.sender.hostWebContents);
    var menu = electron.Menu.buildFromTemplate(menuDefinition);
    menu.popup(win);
    resolve(selection);
  })
}

function openFolder (folderPath) {
  electron.shell.openExternal('file://' + folderPath);
}

async function getDefaultLocalPath (dir, title) {
  // massage the title
  title = typeof title === 'string' ? title : '';
  title = title.replace(_beaker_core_lib_const.INVALID_SAVE_FOLDER_CHAR_REGEX, '');
  if (!title.trim()) {
    title = 'Untitled';
  }
  title = slugify(title).toLowerCase();

  // find an available variant of title
  let tryNum = 1;
  let titleVariant = title;
  while (await jetpack.existsAsync(path__default.join(dir, titleVariant))) {
    titleVariant = `${title}-${++tryNum}`;
  }
  return path__default.join(dir, titleVariant)
}

async function doWebcontentsCmd (method, wcId, ...args) {
  var wc = electron.webContents.fromId(+wcId);
  if (!wc) throw new Error(`WebContents not found (${wcId})`)
  return wc[method](...args)
}

async function doTest (test) {
}

// internal methods
// =

function setUpdaterState (state) {
  updaterState = state;
  browserEvents.emit('updater-state-changed', {state});
}

function getAutoUpdaterFeedSettings () {
  return {
    provider: 'github',
    repo: 'beaker',
    owner: 'beakerbrowser',
    vPrefixedTagName: false
  }
}

// run a daily check for new updates
function scheduledAutoUpdate () {
  settingsDb.get('auto_update_enabled').then(v => {
    // if auto updates are enabled, run the check
    if (+v === 1) { checkForUpdates(); }

    // schedule next check
    setTimeout(scheduledAutoUpdate, SCHEDULED_AUTO_UPDATE_DELAY);
  });
}

// event handlers
// =

function onUpdateAvailable () {
  debug('[AUTO-UPDATE] New version available. Downloading...');
  electronUpdater.autoUpdater.downloadUpdate();
  setUpdaterState(UPDATER_STATUS_DOWNLOADING);
}

function onUpdateNotAvailable () {
  debug('[AUTO-UPDATE] No browser update available.');
  setUpdaterState(UPDATER_STATUS_IDLE);
}

function onUpdateDownloaded () {
  debug('[AUTO-UPDATE] New browser version downloaded. Ready to install.');
  setUpdaterState(UPDATER_STATUS_DOWNLOADED);
}

function onUpdateError (e) {
  console.error(e);
  debug('[AUTO-UPDATE] error', e.toString(), e);
  setUpdaterState(UPDATER_STATUS_IDLE);
  updaterError = {message: (e.toString() || '').split('\n')[0]};
  browserEvents.emit('updater-error', updaterError);
}

function onWebContentsCreated (e, webContents$$1) {
  webContents$$1.on('will-prevent-unload', onWillPreventUnload);
  webContents$$1.on('remote-require', e => {
    // do not allow
    e.preventDefault();
  });
  webContents$$1.on('remote-get-global', e => {
    // do not allow
    e.preventDefault();
  });
}

async function onWillPreventUnload (e) {
  var {response} = await electron.dialog.showMessageBox({
    type: 'question',
    buttons: ['Leave', 'Stay'],
    title: 'Do you want to leave this site?',
    message: 'Changes you made may not be saved.',
    defaultId: 0,
    cancelId: 1
  });
  var leave = (response === 0);
  if (leave) {
    e.preventDefault();
  }
}

function onCompleted (details) {
  function set (v) {
    resourceContentTypes.set(details.url, Array.isArray(v) ? v[0] : v);
  }
  if (!details.responseHeaders) return
  if ('Content-Type' in details.responseHeaders) {
    set(details.responseHeaders['Content-Type']);
  } else if ('content-type' in details.responseHeaders) {
    set(details.responseHeaders['content-type']);
  }
}

// import { initialize, containsAds } from 'contains-ads'

// exported API

function setup$13 () {
  /*
  temporarily disabled due to https://github.com/bbondy/bloom-filter-cpp/issues/7
    "contains-ads": "^0.2.5",

  initialize() // contains-ads

  const beakerUrls = /^(beaker|blob)/
  session.defaultSession.webRequest.onBeforeRequest(['*://*./*'], (details, callback) => {
    const shouldBeBlocked = !details.url.match(beakerUrls) && containsAds(details.url)

    if (shouldBeBlocked) {
      callback({cancel: true})
    } else {
      callback({cancel: false})
    }
  })
  */
}

const settingsDb$3 = beakerCore.dbs.settings;
const debug$1 = beakerCore.debugLogger('beaker-analytics');

// exported methods
// =

function setup$14 () {
  setTimeout(checkin, ms('3s'));
}

// internal methods
// =

async function checkin () {
  // enabled?
  var isEnabled = await settingsDb$3.get('analytics_enabled');
  if (isEnabled == 1) {
    try {
      var pingData = await readPingData();
      if ((Date.now() - (pingData.lastPingTime || 0)) > _beaker_core_lib_const.ANALYTICS_CHECKIN_INTERVAL) {
        await sendPing(pingData);
        pingData.lastPingTime = Date.now();
        await writePingData(pingData);
      }
    } catch (e) {
      // failed, we'll reschedule another ping in 24 hours
    }
  }

  // schedule another ping check in 24 hours
  var to = setTimeout(checkin, ms('1d'));
  to.unref();
}

function sendPing (pingData) {
  return new Promise((resolve, reject) => {
    var qs = querystring.stringify({userId: pingData.id, os: osName(), beakerVersion: electron.app.getVersion()});
    debug$1('Sending ping to %s: %s', _beaker_core_lib_const.ANALYTICS_SERVER, qs);

    var req = https.request({
      method: 'POST',
      hostname: _beaker_core_lib_const.ANALYTICS_SERVER,
      path: '/ping?' + qs
    }, (res) => {
      if (res.statusCode === 204) {
        debug$1('Ping succeeded');
        resolve();
      } else {
        res.setEncoding('utf8');
        res.pipe(concat(body => debug$1('Ping failed', res.statusCode, body)));
        reject();
      }
    });
    req.on('error', err => {
      debug$1('Ping failed', err);
      reject();
    });
    req.end();
  })
}

async function readPingData () {
  var data = await jetpack.readAsync(path__default.join(electron.app.getPath('userData'), _beaker_core_lib_const.ANALYTICS_DATA_FILE), 'json');
  return data || {lastPingTime: 0, id: crypto.randomBytes(32).toString('hex')}
}

async function writePingData (data) {
  return jetpack.writeAsync(path__default.join(electron.app.getPath('userData'), _beaker_core_lib_const.ANALYTICS_DATA_FILE), data)
}

//import { app } from 'electron'
// exported methods
// =

function setup$15 () {
  setTimeout(openPort, ms('3s'));
}

function closePort () {
  var client = natUpnp.createClient();
  client.portUnmapping({public: _beaker_core_lib_const.DAT_SWARM_PORT});
}

// internal methods
// =

async function openPort () {
  var opts = {
    public: _beaker_core_lib_const.DAT_SWARM_PORT,
    private: _beaker_core_lib_const.DAT_SWARM_PORT,
    ttl: 1800  // 30 min
  };

  var client = natUpnp.createClient();
  client.portMapping(opts, async (err) => {
    if (!err) {
      // schedule reopening the port every 30 minutes
      var to = setTimeout(openPort, ms('30m'));
      to.unref();
    } else {
      console.log(err);
      // assuming errorCode 725 OnlyPermanentLeasesSupported and retry without a TTL
      opts.ttl = '0';  // string not int
      client.portMapping(opts);
    }
  });
}

function registerContextMenu () {
  // register the context menu on every created webContents
  electron.app.on('web-contents-created', (e, webContents$$1) => {
    webContents$$1.on('context-menu', async (e, props) => {
      var menuItems = [];
      const { mediaFlags, editFlags } = props;
      const hasText = props.selectionText.trim().length > 0;
      const can = type => editFlags[`can${type}`] && hasText;
      const isDat = props.pageURL.startsWith('dat://');

      // get the focused window, ignore if not available (not in focus)
      // - fromWebContents(webContents) doesnt seem to work, maybe because webContents is often a webview?
      var targetWindow = electron.BrowserWindow.getFocusedWindow();
      if (!targetWindow) { return }

      // ignore clicks on the shell window
      if (props.pageURL == 'beaker://shell-window/') { return }

      // helper to call code on the element under the cursor
      const callOnElement = js => webContents$$1.executeJavaScript(`
        var el = document.elementFromPoint(${props.x}, ${props.y})
        new Promise(resolve => { ${js} })
      `);

      // helper to run a download prompt for media
      const downloadPrompt = (field, ext) => async (item, win) => {
        var defaultPath = path__default.join(electron.app.getPath('downloads'), path__default.basename(props[field]));
        if (ext && defaultPath.split('/').pop().indexOf('.') === -1) defaultPath += ext;
        var {filePath} = await electron.dialog.showSaveDialog({ title: `Save ${props.mediaType} as...`, defaultPath });
        if (filePath) { download(win, webContents$$1, props[field], { saveAs: filePath }); }
      };

      // links
      if (props.linkURL && props.mediaType === 'none') {
        menuItems.push({ label: 'Open Link in New Tab', click: (item, win) => create$$1(win, props.linkURL) });
        menuItems.push({ label: 'Copy Link Address', click: () => electron.clipboard.writeText(props.linkURL) });
        menuItems.push({ type: 'separator' });
      }

      // images
      if (props.mediaType == 'image') {
        menuItems.push({ label: 'Save Image As...', click: downloadPrompt('srcURL') });
        menuItems.push({ label: 'Copy Image', click: () => webContents$$1.copyImageAt(props.x, props.y) });
        menuItems.push({ label: 'Copy Image URL', click: () => electron.clipboard.writeText(props.srcURL) });
        menuItems.push({ label: 'Open Image in New Tab', click: (item, win) => create$$1(win, props.srcURL) });
        menuItems.push({ type: 'separator' });
      }

      // videos and audios
      if (props.mediaType == 'video' || props.mediaType == 'audio') {
        menuItems.push({ label: 'Loop', type: 'checkbox', checked: mediaFlags.isLooping, click: () => callOnElement('el.loop = !el.loop') });
        if (mediaFlags.hasAudio) { menuItems.push({ label: 'Muted', type: 'checkbox', checked: mediaFlags.isMuted, click: () => callOnElement('el.muted = !el.muted') }); }
        if (mediaFlags.canToggleControls) { menuItems.push({ label: 'Show Controls', type: 'checkbox', checked: mediaFlags.isControlsVisible, click: () => callOnElement('el.controls = !el.controls') }); }
        menuItems.push({ type: 'separator' });
      }

      // videos
      if (props.mediaType == 'video') {
        menuItems.push({ label: 'Save Video As...', click: downloadPrompt('srcURL') });
        menuItems.push({ label: 'Copy Video URL', click: () => electron.clipboard.writeText(props.srcURL) });
        menuItems.push({ label: 'Open Video in New Tab', click: (item, win) => create$$1(win, props.srcURL) });
        menuItems.push({ type: 'separator' });
      }

      // audios
      if (props.mediaType == 'audio') {
        menuItems.push({ label: 'Save Audio As...', click: downloadPrompt('srcURL') });
        menuItems.push({ label: 'Copy Audio URL', click: () => electron.clipboard.writeText(props.srcURL) });
        menuItems.push({ label: 'Open Audio in New Tab', click: (item, win) => create$$1(win, props.srcURL) });
        menuItems.push({ type: 'separator' });
      }

      // clipboard
      if (props.isEditable) {
        menuItems.push({ label: 'Cut', role: 'cut', enabled: can('Cut') });
        menuItems.push({ label: 'Copy', role: 'copy', enabled: can('Copy') });
        menuItems.push({ label: 'Paste', role: 'paste', enabled: editFlags.canPaste });
        menuItems.push({ type: 'separator' });
      } else if (hasText) {
        menuItems.push({ label: 'Copy', role: 'copy', enabled: can('Copy') });
        menuItems.push({ type: 'separator' });
      }

      // web search
      if (hasText) {
        var searchPreviewStr = props.selectionText.substr(0, 30); // Trim search preview to keep it reasonably sized
        searchPreviewStr = searchPreviewStr.replace(/\s/gi, ' '); // Replace whitespace chars with space
        searchPreviewStr = searchPreviewStr.replace(/[\u061c\u200E\u200f\u202A-\u202E]+/g, ''); // Remove directional text control chars
        if (searchPreviewStr.length < props.selectionText.length) { // Add ellipsis if search preview was trimmed
          searchPreviewStr += '..."';
        } else {
          searchPreviewStr += '"';
        }
        var query = 'https://duckduckgo.com/?q=' + encodeURIComponent(props.selectionText.substr(0, 500)); // Limit query to prevent too long query error from DDG
        menuItems.push({ label: 'Search DuckDuckGo for "' + searchPreviewStr, click: (item, win) => create$$1(win, query) });
        menuItems.push({ type: 'separator' });
      }

      if (!props.linkURL && props.mediaType === 'none' && !hasText) {
        menuItems.push({
          label: 'Back',
          enabled: webContents$$1.canGoBack(),
          click: () => webContents$$1.goBack()
        });
        menuItems.push({
          label: 'Forward',
          enabled: webContents$$1.canGoForward(),
          click: () => webContents$$1.goForward()
        });
        menuItems.push({
          label: 'Reload',
          click: () => webContents$$1.reload()
        });
        menuItems.push({ type: 'separator' });
        menuItems.push({
          label: 'Save Page As...',
          click: downloadPrompt('pageURL', '.html')
        });
        menuItems.push({
          label: 'Print...',
          click: () => webContents$$1.print()
        });
        menuItems.push({ type: 'separator' });
        if (isDat) {
          menuItems.push({
            label: 'View Source',
            click: (item, win) => {
              create$$1(win, 'beaker://library/' + props.pageURL);
            }
          });
          menuItems.push({
            label: 'Network Debugger',
            click: (item, win) => {
              create$$1(win, 'beaker://swarm-debugger/' + props.pageURL);
            }
          });
          menuItems.push({ type: 'separator' });
        }
      }

      // inspector
      menuItems.push({
        label: 'Inspect Element',
        click: item => {
          webContents$$1.inspectElement(props.x, props.y);
          if (webContents$$1.isDevToolsOpened()) { webContents$$1.devToolsWebContents.focus(); }
        }
      });

      // show menu
      var menu = electron.Menu.buildFromTemplate(menuItems);
      menu.popup({ window: targetWindow });
    });
  });
}

// globals
// =

var processes = {};
process.on('exit', closeAll);

// exported api
// =

async function spawn (id, modulePath) {
  var fullModulePath = require.resolve(modulePath);
  const opts = {
    stdio: 'inherit',
    env: Object.assign({}, process.env, {
      ELECTRON_RUN_AS_NODE: 1
    })
  };
  console.log(`Starting external process ${id}`);
  var proc = childProcess.fork(fullModulePath, opts)
    .on('error', console.log)
    .on('close', () => {
      delete processes[id];
    });

  proc.send({
    userDataPath: electron.app.getPath('userData')
  });

  await new Promise((resolve, reject) => {
    proc.once('message', resolve);
  });

  processes[id] = proc;
  return proc
}

function close$2 (id) {
  processes[id].kill();
  delete processes[id];
}

function closeAll () {
  for (let id in processes) {
    close$2(id);
  }
}

const {templates} = beakerCore.dbs;
const {archivesDebugPage, datDnsCachePage, datDnsCacheJS} = beakerCore.dat.debug;
// constants
// =

// content security policies
const BEAKER_CSP = `
  default-src 'self' beaker:;
  img-src beaker-favicon: beaker: data: dat: http: https;
  script-src 'self' beaker: 'unsafe-eval';
  media-src 'self' beaker: dat:;
  style-src 'self' 'unsafe-inline' beaker:;
  child-src 'self';
`.replace(/\n/g, '');

// exported api
// =

function setup$16 () {
  // setup the protocol handler
  electron.protocol.registerStreamProtocol('beaker', beakerProtocol, err => {
    if (err) throw new Error('Failed to create protocol: beaker. ' + err)
  });
}

// internal methods
// =

async function beakerProtocol (request, respond) {
  var cb = once((statusCode, status, contentType, path$$1) => {
    const headers = {
      'Cache-Control': 'no-cache',
      'Content-Type': (contentType || 'text/html; charset=utf-8'),
      'Content-Security-Policy': BEAKER_CSP,
      'Access-Control-Allow-Origin': '*'
    };
    if (typeof path$$1 === 'string') {
      respond({statusCode, headers, data: fs__default.createReadStream(path$$1)});
    } else if (typeof path$$1 === 'function') {
      respond({statusCode, headers, data: intoStream(path$$1())});
    } else {
      respond({statusCode, headers, data: intoStream(errorPage(statusCode + ' ' + status))});
    }
  });
  async function serveICO (path$$1, size = 16) {
    // read the file
    const data = await jetpack.readAsync(path$$1, 'buffer');

    // parse the ICO to get the 16x16
    const images = await ICO.parse(data, 'image/png');
    let image = images[0];
    for (let i = 1; i < images.length; i++) {
      if (Math.abs(images[i].width - size) < Math.abs(image.width - size)) {
        image = images[i];
      }
    }

    // serve
    cb(200, 'OK', 'image/png', () => Buffer.from(image.buffer));
  }

  var requestUrl = request.url;
  var queryParams;
  {
    // strip off the hash
    let i = requestUrl.indexOf('#');
    if (i !== -1) requestUrl = requestUrl.slice(0, i);
  }
  {
    // get the query params
    queryParams = url__default.parse(requestUrl, true).query;

    // strip off the query
    let i = requestUrl.indexOf('?');
    if (i !== -1) requestUrl = requestUrl.slice(0, i);
  }

  // browser ui
  if (requestUrl === 'beaker://shell-window/') {
    return cb(200, 'OK', 'text/html; charset=utf-8', path__default.join(__dirname, 'shell-window.html'))
  }
  if (requestUrl === 'beaker://shell-window/main.js') {
    return cb(200, 'OK', 'application/javascript; charset=utf-8', path__default.join(__dirname, 'shell-window.build.js'))
  }
  if (requestUrl === 'beaker://shell-window/main.css') {
    return cb(200, 'OK', 'text/css; charset=utf-8', path__default.join(__dirname, 'stylesheets/shell-window.css'))
  }
  if (requestUrl === 'beaker://location-bar/') {
    return cb(200, 'OK', 'text/html; charset=utf-8', path__default.join(__dirname, 'location-bar.html'))
  }
  if (requestUrl === 'beaker://shell-menus/') {
    return cb(200, 'OK', 'text/html; charset=utf-8', path__default.join(__dirname, 'shell-menus.html'))
  }
  if (requestUrl === 'beaker://perm-prompt/') {
    return cb(200, 'OK', 'text/html; charset=utf-8', path__default.join(__dirname, 'perm-prompt.html'))
  }
  if (requestUrl === 'beaker://modals/') {
    return cb(200, 'OK', 'text/html; charset=utf-8', path__default.join(__dirname, 'modals.html'))
  }
  if (requestUrl === 'beaker://assets/syntax-highlight.js') {
    return cb(200, 'OK', 'application/javascript; charset=utf-8', path__default.join(__dirname, 'assets/js/syntax-highlight.js'))
  }
  if (requestUrl === 'beaker://assets/syntax-highlight.css') {
    return cb(200, 'OK', 'text/css; charset=utf-8', path__default.join(__dirname, 'assets/css/syntax-highlight.css'))
  }
  if (requestUrl === 'beaker://assets/icons.css') {
    return cb(200, 'OK', 'text/css; charset=utf-8', path__default.join(__dirname, 'stylesheets/icons.css'))
  }
  if (requestUrl === 'beaker://assets/font-awesome.css') {
    return cb(200, 'OK', 'text/css; charset=utf-8', path__default.join(__dirname, 'stylesheets/fonts/font-awesome/css/all.min.css'))
  }
  if (requestUrl === 'beaker://assets/fa-regular-400.woff2') {
    return cb(200, 'OK', 'text/css; charset=utf-8', path__default.join(__dirname, 'assets/fonts/fa-regular-400.woff2'))
  }
  if (requestUrl === 'beaker://assets/fa-regular-400.woff') {
    return cb(200, 'OK', 'text/css; charset=utf-8', path__default.join(__dirname, 'assets/fonts/fa-regular-400.woff'))
  }
  if (requestUrl === 'beaker://assets/fa-regular-400.svg') {
    return cb(200, 'OK', 'text/css; charset=utf-8', path__default.join(__dirname, 'assets/fonts/fa-regular-400.svg'))
  }
  if (requestUrl === 'beaker://assets/fa-solid-900.woff2') {
    return cb(200, 'OK', 'text/css; charset=utf-8', path__default.join(__dirname, 'assets/fonts/fa-solid-900.woff2'))
  }
  if (requestUrl === 'beaker://assets/fa-solid-900.woff') {
    return cb(200, 'OK', 'text/css; charset=utf-8', path__default.join(__dirname, 'assets/fonts/fa-solid-900.woff'))
  }
  if (requestUrl === 'beaker://assets/fa-solid-900.svg') {
    return cb(200, 'OK', 'text/css; charset=utf-8', path__default.join(__dirname, 'assets/fonts/fa-solid-900.svg'))
  }
  if (requestUrl === 'beaker://assets/fa-brands-400.woff2') {
    return cb(200, 'OK', 'text/css; charset=utf-8', path__default.join(__dirname, 'assets/fonts/fa-brands-400.woff2'))
  }
  if (requestUrl === 'beaker://assets/fa-brands-400.woff') {
    return cb(200, 'OK', 'text/css; charset=utf-8', path__default.join(__dirname, 'assets/fonts/fa-brands-400.woff'))
  }
  if (requestUrl === 'beaker://assets/fa-brands-400.svg') {
    return cb(200, 'OK', 'text/css; charset=utf-8', path__default.join(__dirname, 'assets/fonts/fa-brands-400.svg'))
  }
  if (requestUrl === 'beaker://assets/font-photon-entypo') {
    return cb(200, 'OK', 'application/font-woff', path__default.join(__dirname, 'assets/fonts/photon-entypo.woff'))
  }
  if (requestUrl === 'beaker://assets/font-source-sans-pro') {
    return cb(200, 'OK', 'application/font-woff2', path__default.join(__dirname, 'assets/fonts/source-sans-pro.woff2'))
  }
  if (requestUrl === 'beaker://assets/font-source-sans-pro-le') {
    return cb(200, 'OK', 'application/font-woff2', path__default.join(__dirname, 'assets/fonts/source-sans-pro-le.woff2'))
  }
  if (requestUrl.startsWith('beaker://assets/logo2')) {
    return cb(200, 'OK', 'image/png', path__default.join(__dirname, 'assets/img/logo2.png'))
  }
  if (requestUrl.startsWith('beaker://assets/logo')) {
    return cb(200, 'OK', 'image/png', path__default.join(__dirname, 'assets/img/logo.png'))
  }
  if (requestUrl.startsWith('beaker://assets/favicons/')) {
    return serveICO(path__default.join(__dirname, 'assets/favicons', requestUrl.slice('beaker://assets/favicons/'.length)))
  }

  // template screenshots
  if (requestUrl.startsWith('beaker://templates/screenshot/')) {
    let templateUrl = requestUrl.slice('beaker://templates/screenshot/'.length);
    templates.getScreenshot(0, templateUrl)
      .then(({screenshot}) => {
        screenshot = screenshot.split(',')[1];
        cb(200, 'OK', 'image/png', () => Buffer.from(screenshot, 'base64'));
      })
      .catch(err => {
        console.error('Failed to load template screenshot', templateUrl, err);
        return cb(404, 'Not Found')
      });
    return
  }

  // builtin pages
  if (requestUrl === 'beaker://assets/builtin-pages.css') {
    return cb(200, 'OK', 'text/css; charset=utf-8', path__default.join(__dirname, 'stylesheets/builtin-pages.css'))
  }
  if (requestUrl.startsWith('beaker://assets/img/onboarding/')) {
    let imgPath = requestUrl.slice('beaker://assets/img/onboarding/'.length);
    return cb(200, 'OK', 'image/svg+xml', path__default.join(__dirname, `assets/img/onboarding/${imgPath}`))
  }
  if (requestUrl.startsWith('beaker://assets/img/templates/')) {
    let imgPath = requestUrl.slice('beaker://assets/img/templates/'.length);
    return cb(200, 'OK', 'image/png', path__default.join(__dirname, `assets/img/templates/${imgPath}`))
  }
  if (requestUrl.startsWith('beaker://assets/ace/') && requestUrl.endsWith('.js')) {
    let filePath = requestUrl.slice('beaker://assets/ace/'.length);
    return cb(200, 'OK', 'application/javascript', path__default.join(__dirname, `assets/js/ace-1.3.3/${filePath}`))
  }
  if (requestUrl === 'beaker://assets/icon/photos.png') {
    return cb(200, 'OK', 'image/png', path__default.join(__dirname, 'assets/img/icon/photos.png'))
  }
  if (requestUrl === 'beaker://assets/icon/avatar.svg') {
    return cb(200, 'OK', 'image/svg+xml', path__default.join(__dirname, 'assets/img/icon/avatar.svg'))
  }
  if (requestUrl === 'beaker://assets/icon/folder-color.png') {
    return cb(200, 'OK', 'image/png', path__default.join(__dirname, 'assets/img/icon/folder-color.png'))
  }
  if (requestUrl === 'beaker://assets/icon/grid.svg') {
    return cb(200, 'OK', 'image/svg+xml', path__default.join(__dirname, 'assets/img/icon/grid.svg'))
  }
  if (requestUrl === 'beaker://assets/icon/star.svg') {
    return cb(200, 'OK', 'image/svg+xml', path__default.join(__dirname, 'assets/img/icon/star.svg'))
  }
  if (requestUrl === 'beaker://assets/icon/filesystem.svg') {
    return cb(200, 'OK', 'image/svg+xml', path__default.join(__dirname, 'assets/img/icon/filesystem.svg'))
  }
  if (requestUrl === 'beaker://assets/icon/history.svg') {
    return cb(200, 'OK', 'image/svg+xml', path__default.join(__dirname, 'assets/img/icon/history.svg'))
  }
  if (requestUrl === 'beaker://assets/icon/gear.svg') {
    return cb(200, 'OK', 'image/svg+xml', path__default.join(__dirname, 'assets/img/icon/gear.svg'))
  }
  if (requestUrl === 'beaker://start/') {
    return cb(200, 'OK', 'text/html; charset=utf-8', path__default.join(__dirname, 'builtin-pages/start.html'))
  }
  if (requestUrl.startsWith('beaker://start/background-image-default')) {
    let imgPath = requestUrl.slice('beaker://start/background-image-default'.length);
    return cb(200, 'OK', 'image/png', path__default.join(__dirname, `assets/img/start${imgPath}`))
  }
  if (requestUrl === 'beaker://start/background-image') {
    return cb(200, 'OK', 'image/png', path__default.join(electron.app.getPath('userData'), 'start-background-image'))
  }
  if (requestUrl === 'beaker://start/main.css') {
    return cb(200, 'OK', 'text/css; charset=utf-8', path__default.join(__dirname, 'stylesheets/builtin-pages/start.css'))
  }
  if (requestUrl === 'beaker://start/main.js') {
    return cb(200, 'OK', 'application/javascript; charset=utf-8', path__default.join(__dirname, 'builtin-pages/build/start.build.js'))
  }
  if (requestUrl === 'beaker://profile/main.js') {
    return cb(200, 'OK', 'application/javascript; charset=utf-8', path__default.join(__dirname, 'builtin-pages/build/profile.build.js'))
  }
  if (requestUrl === 'beaker://profile/' || requestUrl.startsWith('beaker://profile/')) {
    return cb(200, 'OK', 'text/html; charset=utf-8', path__default.join(__dirname, 'builtin-pages/profile.html'))
  }
  if (requestUrl === 'beaker://bookmarks/') {
    return cb(200, 'OK', 'text/html; charset=utf-8', path__default.join(__dirname, 'builtin-pages/bookmarks.html'))
  }
  if (requestUrl === 'beaker://bookmarks/main.js') {
    return cb(200, 'OK', 'application/javascript; charset=utf-8', path__default.join(__dirname, 'builtin-pages/build/bookmarks.build.js'))
  }
  if (requestUrl === 'beaker://history/') {
    return cb(200, 'OK', 'text/html; charset=utf-8', path__default.join(__dirname, 'builtin-pages/history.html'))
  }
  if (requestUrl === 'beaker://history/main.js') {
    return cb(200, 'OK', 'application/javascript; charset=utf-8', path__default.join(__dirname, 'builtin-pages/build/history.build.js'))
  }
  if (requestUrl === 'beaker://downloads/') {
    return cb(200, 'OK', 'text/html; charset=utf-8', path__default.join(__dirname, 'builtin-pages/downloads.html'))
  }
  if (requestUrl === 'beaker://downloads/main.js') {
    return cb(200, 'OK', 'application/javascript; charset=utf-8', path__default.join(__dirname, 'builtin-pages/build/downloads.build.js'))
  }
  // if (requestUrl === 'beaker://filesystem/main.css') {
  //   return cb(200, 'OK', 'text/css; charset=utf-8', path.join(__dirname, 'stylesheets/builtin-pages/filesystem.css'))
  // }
  // if (requestUrl === 'beaker://filesystem/main.js') {
  //   return cb(200, 'OK', 'application/javascript; charset=utf-8', path.join(__dirname, 'builtin-pages/build/filesystem.build.js'))
  // }
  // if (requestUrl === 'beaker://filesystem/' || requestUrl.startsWith('beaker://filesystem/')) {
  //   return cb(200, 'OK', 'text/html; charset=utf-8', path.join(__dirname, 'builtin-pages/filesystem.html'))
  // }
  if (requestUrl === 'beaker://library/main.css') {
    return cb(200, 'OK', 'text/css; charset=utf-8', path__default.join(__dirname, 'stylesheets/builtin-pages/library.css'))
  }
  if (requestUrl === 'beaker://library/main.js') {
    return cb(200, 'OK', 'application/javascript; charset=utf-8', path__default.join(__dirname, 'builtin-pages/build/library.build.js'))
  }
  if (requestUrl === 'beaker://library/view.js') {
    return cb(200, 'OK', 'application/javascript; charset=utf-8', path__default.join(__dirname, 'builtin-pages/build/library-view.build.js'))
  }
  if (requestUrl === 'beaker://library/') {
    return cb(200, 'OK', 'text/html; charset=utf-8', path__default.join(__dirname, 'builtin-pages/library.html'))
  }
  if (requestUrl.startsWith('beaker://library/')) {
    return cb(200, 'OK', 'text/html; charset=utf-8', path__default.join(__dirname, 'builtin-pages/library-view.html'))
  }
  // if (requestUrl === 'beaker://install-modal/main.css') {
  //   return cb(200, 'OK', 'text/css; charset=utf-8', path.join(__dirname, 'stylesheets/builtin-pages/install-modal.css'))
  // }
  // if (requestUrl === 'beaker://install-modal/main.js') {
  //   return cb(200, 'OK', 'application/javascript; charset=utf-8', path.join(__dirname, 'builtin-pages/build/install-modal.build.js'))
  // }
  // if (requestUrl === 'beaker://install-modal/' || requestUrl.startsWith('beaker://install-modal/')) {
  //   return cb(200, 'OK', 'text/html; charset=utf-8', path.join(__dirname, 'builtin-pages/install-modal.html'))
  // }
  if (requestUrl === 'beaker://view-source/main.css') {
    return cb(200, 'OK', 'text/css; charset=utf-8', path__default.join(__dirname, 'stylesheets/builtin-pages/view-source.css'))
  }
  if (requestUrl === 'beaker://view-source/main.js') {
    return cb(200, 'OK', 'application/javascript; charset=utf-8', path__default.join(__dirname, 'builtin-pages/build/view-source.build.js'))
  }
  if (requestUrl === 'beaker://view-source/' || requestUrl.startsWith('beaker://view-source/')) {
    return cb(200, 'OK', 'text/html; charset=utf-8', path__default.join(__dirname, 'builtin-pages/view-source.html'))
  }
  if (requestUrl === 'beaker://swarm-debugger/main.css') {
    return cb(200, 'OK', 'text/css; charset=utf-8', path__default.join(__dirname, 'stylesheets/builtin-pages/swarm-debugger.css'))
  }
  if (requestUrl === 'beaker://swarm-debugger/main.js') {
    return cb(200, 'OK', 'application/javascript; charset=utf-8', path__default.join(__dirname, 'builtin-pages/build/swarm-debugger.build.js'))
  }
  if (requestUrl === 'beaker://swarm-debugger/' || requestUrl.startsWith('beaker://swarm-debugger/')) {
    return cb(200, 'OK', 'text/html; charset=utf-8', path__default.join(__dirname, 'builtin-pages/swarm-debugger.html'))
  }
  if (requestUrl === 'beaker://settings/') {
    return cb(200, 'OK', 'text/html; charset=utf-8', path__default.join(__dirname, 'builtin-pages/settings.html'))
  }
  if (requestUrl === 'beaker://settings/main.js') {
    return cb(200, 'OK', 'application/javascript; charset=utf-8', path__default.join(__dirname, 'builtin-pages/build/settings.build.js'))
  }
  if (requestUrl === 'beaker://watchlist/main.css') {
    return cb(200, 'OK', 'text/css; charset=utf-8', path__default.join(__dirname, 'stylesheets/builtin-pages/watchlist.css'))
  }
  if (requestUrl === 'beaker://watchlist/main.js') {
    return cb(200, 'OK', 'application/javascript; charset=utf-8', path__default.join(__dirname, 'builtin-pages/build/watchlist.build.js'))
  }
  if (requestUrl === 'beaker://watchlist/') {
    return cb(200, 'OK', 'text/html; charset=utf-8', path__default.join(__dirname, 'builtin-pages/watchlist.html'))
  }

  // debugging
  if (requestUrl === 'beaker://internal-archives/') {
    return cb(200, 'OK', 'text/html; charset=utf-8', archivesDebugPage)
  }
  if (requestUrl === 'beaker://dat-dns-cache/') {
    return cb(200, 'OK', 'text/html; charset=utf-8', datDnsCachePage)
  }
  if (requestUrl === 'beaker://dat-dns-cache/main.js') {
    return cb(200, 'OK', 'application/javascript; charset=utf-8', datDnsCacheJS)
  }
  if (requestUrl.startsWith('beaker://debug-log/')) {
    const PAGE_SIZE = 1e6;
    var start = queryParams.start ? (+queryParams.start) : 0;
    let content = await beakerCore.getLogFileContent(start, start + PAGE_SIZE);
    var pagination = `<h2>Showing bytes ${start} - ${start + PAGE_SIZE}. <a href="beaker://debug-log/?start=${start + PAGE_SIZE}">Next page</a></h2>`;
    return respond({
      statusCode: 200,
      headers: {
        'Content-Type': 'text/html; charset=utf-8',
        'Content-Security-Policy': BEAKER_CSP,
        'Access-Control-Allow-Origin': '*'
      },
      data: intoStream(`
        ${pagination}
        <pre>${content.replace(/</g, '&lt;').replace(/>/g, '&gt;')}</pre>
        ${pagination}
      `)
    })
  }

  return cb(404, 'Not Found')
}

/**
 * beaker-favicon:
 *
 * Helper protocol to serve site favicons from the sitedata db.
 **/

const {dat: dat$2} = beakerCore;
const {sitedata: sitedata$1} = beakerCore.dbs;
function setup$17 () {
  // load default favicon
  var defaultFaviconBuffer = -6; // not found, till we load it
  fs__default.readFile(path__default.join(__dirname, './assets/img/default-favicon.png'), (err, buf) => {
    if (err) { console.error('Failed to load default favicon', path__default.join(__dirname, './assets/img/default-favicon.png'), err); }
    if (buf) { defaultFaviconBuffer = buf; }
  });

  // detect if is retina
  let display = electron.screen.getPrimaryDisplay();
  const isRetina = display.scaleFactor >= 2;

  // register favicon protocol
  electron.protocol.registerBufferProtocol('beaker-favicon', async (request, cb) => {
    // parse the URL
    let {url: url$$1, faviconSize} = parseBeakerFaviconURL(request.url);
    if (isRetina) {
      faviconSize *= 2;
    }

    // if a dat, see if there's a favicon.ico or .png
    try {
      let data, fs$$1;
      // pick the filesystem
      let datResolvedUrl = url$$1;
      if (url$$1.startsWith('dat://')) {
        datResolvedUrl = await dat$2.dns.resolveName(url$$1);
        fs$$1 = dat$2.library.getArchive(datResolvedUrl); // (only try if the dat is loaded)
      }
      if (fs$$1) {
        // try .ico
        try {
          data = await pda.readFile(fs$$1, '/favicon.ico', 'binary');
          if (data) {
            // select the best-fitting size
            let images = await ICO.parse(data, 'image/png');
            let image = images[0];
            for (let i = 1; i < images.length; i++) {
              if (Math.abs(images[i].width - faviconSize) < Math.abs(image.width - faviconSize)) {
                image = images[i];
              }
            }
            let buf = Buffer.from(image.buffer);
            sitedata$1.set(url$$1, 'favicon', `data:image/png;base64,${buf.toString('base64')}`); // cache
            return cb({mimeType: 'image/png', data: buf})
          }
        } catch (e) {
          // .ico failed, ignore
          data = null;
        }

        // try .png
        data = await pda.readFile(fs$$1, '/favicon.png', 'binary');
        if (data) {
          sitedata$1.set(url$$1, 'favicon', `data:image/png;base64,${data.toString('base64')}`); // cache
          return cb({mimeType: 'image/png', data})
        }
      }
    } catch (e) {
      // ignore
    }

    try {
      // look up in db
      let data = await sitedata$1.get(url$$1, 'favicon');
      if (data) {
        // `data` is a data url ('data:image/png;base64,...')
        // so, skip the beginning and pull out the data
        data = data.split(',')[1];
        if (data) {
          return cb({ mimeType: 'image/png', data: Buffer.from(data, 'base64') })
        }
      }
    } catch (e) {
      // ignore
    }

    cb({ mimeType: 'image/png', data: defaultFaviconBuffer });
  }, e => {
    if (e) { console.error('Failed to register beaker-favicon protocol', e); }
  });
}

const BEAKER_FAVICON_URL_RE = /^beaker-favicon:(\d*),?(.*)/;
function parseBeakerFaviconURL (str) {
  const match = BEAKER_FAVICON_URL_RE.exec(str);
  let res = {
    faviconSize: (+match[1]) || 16,
    url: match[2]
  };
  // special case: in beaker://library, use the dat being viewed
  if (res.url.startsWith('beaker://library/dat://')) {
    res.url = res.url.slice('beaker://library/'.length);
  }
  return res
}

const LOG_MESSAGES = false;

var testPort = +beakerCore.getEnvVar('BEAKER_TEST_DRIVER');
var sock;

// exported api
// =

function setup$18 () {
  // setup socket
  sock = dgram.createSocket('udp4');
  sock.bind(0, '127.0.0.1');
  sock.on('message', onMessage);
  sock.on('listening', () => {
    console.log('Test driver enabled, listening for messages on port', sock.address().port);
  });

  // emit ready when ready
  var todos = 2;
  sock.on('listening', hit);
  electron.ipcMain.once('shell-window:ready', hit);
  function hit () {
    if (!(--todos)) {
      // HACK
      // there's some kind of race which causes `executeJavaScript` to not run in the shell window during tests
      // this timeout is intended to solve that
      // -prf
      setTimeout(() => {
        send({isReady: true, port: sock.address().port});
      }, 1e3);
    }
  }
}

// internal methods
// =

function send (obj) {
  if (LOG_MESSAGES) console.log('driverserver sent', JSON.stringify(obj));
  obj = Buffer.from(JSON.stringify(obj), 'utf8');
  sock.send(obj, 0, obj.length, testPort, '127.0.0.1', err => {
    if (err) console.log('Error communicating with the test driver', err);
  });
}

async function onMessage (message) {
  if (LOG_MESSAGES) console.log('driverserver got', message.toString('utf8'));
  const {msgId, cmd, args} = JSON.parse(message.toString('utf8'));
  var method = METHODS[cmd];
  if (!method) method = () => new Error('Invalid method: ' + cmd);
  try {
    var resolve = await method(...args);
    send({msgId, resolve});
  } catch (err) {
    var reject = {
      message: err.message,
      stack: err.stack,
      name: err.name
    };
    send({msgId, reject});
  }
}

const METHODS = {
  newTab () {
    var win = getActiveWindow$1();
    var view = create$$1(win, undefined, {setActive: true});
    return getIndexOfView(win, view)
  },

  navigateTo (page, url$$1) {
    var view = getByIndex(getActiveWindow$1(), page);
    var loadPromise = new Promise(resolve => view.webContents.once('dom-ready', () => resolve()));
    view.loadURL(url$$1);
    return loadPromise
  },

  getUrl (page) {
    var view = getByIndex(getActiveWindow$1(), page);
    return view.url
  },

  async executeJavascriptInShell (js) {
    var win = getActiveWindow$1();
    var res = await win.webContents.executeJavaScript(js);
    return res
  },

  async executeJavascriptOnPage (page, js) {
    var view = getByIndex(getActiveWindow$1(), page);
    var res = await view.webContents.executeJavaScript(js);
    return res
  },

  async executeJavascriptInPermPrompt (page, js) {
    var view = getByIndex(getActiveWindow$1(), page).browserView;
    var prompt = await waitFor(() => get$3(view));
    var res = await prompt.webContents.executeJavaScript(js);
    return res
  },

  async executeJavascriptInModal (page, js) {
    var view = getByIndex(getActiveWindow$1(), page).browserView;
    var modal = await waitFor(() => get$4(view));
    var res = await modal.webContents.executeJavaScript(js);
    return res
  }
};

function getActiveWindow$1 () {
  var win = getActiveWindow();
  while (win.getParentWindow()) {
    win = win.getParentWindow();
  }
  return win
}

function waitFor (condFn) {
  return new Promise(resolve => {
    var i = setInterval(async () => {
      var res = condFn();
      if (res) {
        clearInterval(i);
        return resolve(res)
      }
    }, 100);
  })
}

Error.stackTraceLimit = Infinity;
require('tls').DEFAULT_ECDH_CURVE = 'auto'; // HACK (prf) fix Node 8.9.x TLS issues, see https://github.com/nodejs/node/issues/19359

// This is main process of Electron, started as first thing when your
// app starts. This script is running through entire life of your application.
// It doesn't have any windows which you can see on screen, but we can open
// window from here.

const DISALLOWED_SAVE_PATH_NAMES = [
  'home',
  'desktop',
  'documents',
  'downloads',
  'music',
  'pictures',
  'videos'
];

// setup
// =

// read config from env vars
if (beakerCore.getEnvVar('BEAKER_USER_DATA_PATH')) {
  console.log('User data path set by environment variables');
  console.log('userData:', beakerCore.getEnvVar('BEAKER_USER_DATA_PATH'));
  electron.app.setPath('userData', beakerCore.getEnvVar('BEAKER_USER_DATA_PATH'));
}
if (beakerCore.getEnvVar('BEAKER_TEST_DRIVER')) {
  setup$18();
}

// enable the sandbox
electron.app.enableSandbox();

// configure the protocols
electron.protocol.registerSchemesAsPrivileged([
  {scheme: 'dat', privileges: {standard: true, secure: true, allowServiceWorkers: true, supportFetchAPI: true, corsEnabled: true}},
  {scheme: 'beaker', privileges: {standard: true, secure: true, allowServiceWorkers: true, supportFetchAPI: true, corsEnabled: true}}
]);

// handle OS event to open URLs
electron.app.on('open-url', (e, url$$1) => {
  e.preventDefault(); // we are handling it
  // wait for ready (not waiting can trigger errors)
  if (electron.app.isReady()) open(url$$1);
  else electron.app.on('ready', () => open(url$$1));
});

// handle OS event to open files
electron.app.on('open-file', (e, filepath) => {
  e.preventDefault(); // we are handling it
  // wait for ready (not waiting can trigger errors)
  if (electron.app.isReady()) open(`file://${filepath}`);
  else electron.app.on('ready', () => open(`file://${filepath}`));
});

electron.app.on('ready', async function () {
  // start the daemon process
  var datDaemonProcess = await spawn('dat-daemon', './dat-daemon.js');

  setup$15();

  // setup core
  await beakerCore.setup({
    // paths
    userDataPath: electron.app.getPath('userData'),
    homePath: electron.app.getPath('home'),
    templatesPath: path.join(__dirname, 'assets', 'templates'),
    disallowedSavePaths: DISALLOWED_SAVE_PATH_NAMES.map(path$$1 => electron.app.getPath(path$$1)),

    // APIs
    permsAPI: permissions,
    uiAPI: {
      showModal: create$2,
      capturePage: capturePage
    },
    rpcAPI: rpc,
    downloadsWebAPI: WEBAPI$1,
    browserWebAPI: WEBAPI,
    datDaemonProcess
  });

  // base
  setup$1();
  setup$13();
  setup$14();

  // ui
  setup$10();
  registerContextMenu();
  setup$3();
  setup$11();
  setup$12();

  // protocols
  setup$16();
  setup$17();
  electron.protocol.registerStreamProtocol('dat', beakerCore.dat.protocol.electronHandler, err => {
    if (err) {
      console.error(err);
      throw new Error('Failed to create protocol: dat')
    }
  });
});

electron.app.on('quit', () => {
  closePort();
  closeAll();
});

electron.app.on('custom-ready-to-show', () => {
  // our first window is ready to show, do any additional setup
  beakerCore.dat.library.loadSavedArchives();
});

// only run one instance
const isFirstInstance = electron.app.requestSingleInstanceLock();
if (!isFirstInstance) {
  electron.app.exit();
} else {
  handleArgv(process.argv);
  electron.app.on('second-instance', (event, argv, workingDirectory) => {
    handleArgv(argv);

    // focus/create a window
    ensureOneWindowExists();
    getActiveWindow().focus();
  });
}
function handleArgv (argv) {
  if (process.platform !== 'darwin') {
    // look for URLs, windows & linux use argv instead of open-url
    let url$$1 = argv.find(v => v.indexOf('://') !== -1);
    if (url$$1) {
      open(url$$1);
    }
  }
}

}());